-- This module spies HTR data and save them in the fifo for
-- later retrieval at event building time
library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity HTRstatus is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
        rst_cycle: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        strobe: in STD_LOGIC;
        addr: in STD_LOGIC_VECTOR (15 downto 0);
        we_byte: in STD_LOGIC_VECTOR (3 downto 0);
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        wr_htr_status: in STD_LOGIC;
        en_HTR_bz: in STD_LOGIC;
        htr_status_in: in STD_LOGIC_VECTOR (18 downto 0);
--        htr_status_cntr: out STD_LOGIC_VECTOR (31 downto 0);
        state_LUT: out STD_LOGIC_VECTOR (31 downto 0);
-- HTR states output 
				HTR_ow : out std_logic;
				HTR_bz : out std_logic;
        HTR_state : out std_logic_vector(1 downto 0)
    );
end HTRstatus;

architecture my_arch of HTRstatus is
signal dav_htr_status : std_logic := '0';
signal rd_htr_status : std_logic := '0';
signal htr_ud_cntr_in: std_logic_vector(31 downto 0) := (others => '0');
signal htr_ud_cntr_outp: std_logic_vector(31 downto 0) := (others => '0');
signal htr_ud_cntr_out: std_logic_vector(31 downto 0) := (others => '0');
--signal htr_cntr_in: std_logic_vector(63 downto 0) := (others => '0');
--signal htr_cntr_outp: std_logic_vector(63 downto 0) := (others => '0');
--signal htr_cntr_out: std_logic_vector(63 downto 0) := (others => '0');
signal htr_cntr_wa: std_logic_vector(8 downto 0) := (others => '0');
signal htr_cntr_ra: std_logic_vector(8 downto 0) := (others => '0');
signal wr_cntr: std_logic_vector(3 downto 0) := (others => '0');
--signal rst_cycle : std_logic := '0';
signal htr_status_p : std_logic_vector(18 downto 0) := (others => '0');
signal htr_status : std_logic_vector(15 downto 0) := (others => '0');
signal inc_cntrp : std_logic_vector(7 downto 0) := (others => '0');
signal inc_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal threshold_addra : std_logic_vector(11 downto 0) := (others => '0');
signal threshold_addrb : std_logic_vector(8 downto 0) := (others => '0');
signal stateToGo_addrb : std_logic_vector(8 downto 0) := (others => '0');
signal thresholdp : std_logic_vector(31 downto 0) := (others => '0');
signal threshold : std_logic_vector(31 downto 0) := (others => '0');
signal wr_state : std_logic := '0';
signal wr_stateLUT : std_logic := '0';
signal rst_state_LUT : std_logic := '0';
signal state : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGo_in : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGop : std_logic_vector(31 downto 0) := (others => '0');
signal stateToGo : std_logic_vector(31 downto 0) := (others => '0');
signal merge_state_addr : std_logic_vector(25 downto 0) := (others => '0');
signal merge_state : std_logic_vector(3 downto 0) := (others => '0');
signal gt_threshold : std_logic_vector(7 downto 0) := (others => '0');
signal merge_ow : std_logic_vector(3 downto 0) := (others => '0');
signal merge_bz : std_logic_vector(3 downto 0) := (others => '0');
signal htr_num : std_logic_vector(3 downto 0) := (others => '0');
signal ow_tmp : std_logic := '0';
signal bz_tmp : std_logic := '0';
signal update_ow : std_logic := '0';
-- following are busy and overflow warning status for 15 HTR channels
signal bz : std_logic_vector(15 downto 0) := (others => '0');
signal ow : std_logic_vector(15 downto 0) := (others => '0');
-------------
begin
i_status_que: srl_fifo generic map (N=> 19) PORT MAP(
		rst => reset,
		din => htr_status_in,
		w_strobe => '1',
		we => wr_htr_status,
		clk => clk,
		r_strobe => '1',
		re => rd_htr_status,
		dav => dav_htr_status,
		dout => htr_status_p
	);
-- HTR status counter, modified 5/5/2006
process(clk)
begin
  if(clk'event and clk = '1')then
		if(rd_htr_status = '0' and dav_htr_status = '1')then
			rd_htr_status <= '1';
		else
			rd_htr_status <= '0';
		end if;
    if(reset = '1')then
			htr_cntr_ra(4 downto 1) <= x"0";
		elsif(rd_htr_status = '1')then
			htr_cntr_ra(4 downto 1) <= htr_status_p(18 downto 15);
		elsif(rst_cycle = '1' and htr_cntr_ra(0) = '1')then
			htr_cntr_ra(4 downto 1) <= htr_cntr_ra(4 downto 1) + 1;
		end if;
		htr_cntr_ra(0) <= not htr_cntr_ra(0) and (rd_htr_status or rst_cycle);
		if(rd_htr_status = '1')then
			htr_status <= '0' & htr_status_p(14 downto 0);
		else
			htr_status <= htr_status(7 downto 0) & x"00";
		end if;
		inc_cntrp <= htr_status(15 downto 8);
		inc_cntr <= inc_cntrp;
		wr_cntr <= (wr_cntr(2) or rst_cycle) & wr_cntr(1 downto 0) & ((rd_htr_status or htr_cntr_ra(0)) and not rst_cycle);
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    for i in 0 to 7 loop
      if(inc_cntr(i) = '1' and htr_ud_cntr_out(3+i*4 downto i*4) = threshold(3+i*4 downto i*4))then
				gt_threshold(i) <= '1';
			else
				gt_threshold(i) <= '0';
			end if;
			stateToGo_addrb <= threshold_addrb;
			if(rst_cycle = '1')then
				htr_ud_cntr_in(3+i*4 downto i*4) <= x"0";
      elsif(inc_cntr(i) = '1' and htr_ud_cntr_out(3+i*4 downto i*4) /= threshold(3+i*4 downto i*4))then
				htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4) + 1;
      elsif(inc_cntr(i) = '0' and htr_ud_cntr_out(3+i*4 downto i*4) /= x"0")then
				htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4) - 1;
			else
				htr_ud_cntr_in(3+i*4 downto i*4) <= htr_ud_cntr_out(3+i*4 downto i*4);
			end if;
--			if(rst_cycle = '1')then
--				htr_cntr_in(7+i*8 downto i*8) <= x"00";
--      elsif(inc_cntr(i) = '1' and htr_cntr_out(7+i*8 downto i*8) /= x"ff")then
--				htr_cntr_in(7+i*8 downto i*8) <= htr_cntr_out(7+i*8 downto i*8) + 1;
--			else
--				htr_cntr_in(7+i*8 downto i*8) <= htr_cntr_out(7+i*8 downto i*8);
--			end if;
		end loop;
		htr_ud_cntr_out <= htr_ud_cntr_outp;	 
--		htr_cntr_out <= htr_cntr_outp;	 
		threshold <= thresholdp;	 
		stateToGo <= stateToGop;	 
  end if;
end process;
i_HTRstatus_ud_cntr: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => x"00000000",
  			  DIPA => x"0",
  			  DIB => htr_ud_cntr_in,
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => wr_cntr(3),
  			  SSRA => rst_cycle,
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => htr_cntr_ra,
					ADDRB => htr_cntr_wa,
					DOA => htr_ud_cntr_outp
					);
htr_cntr_ra(8 downto 5) <= "0000";
htr_cntr_wa(8 downto 5) <= "0000";
g_htr_cntr_wa: for i in 0 to 4 generate
	i_htr_cntr_wa: delayN generic map(N => 3) port map(c => clk, d => htr_cntr_ra(i), ce => '1', q => htr_cntr_wa(i));
end generate;
i_threshold: RAMB16_S4_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => w_data(7 downto 4),
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_stateLUT,
  			  WEB => '0',
  			  SSRA => rst_state_LUT,
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => threshold_addra,
					ADDRB => threshold_addrb,
					DOA => state_LUT(7 downto 4),
					DOB => thresholdp
					);
threshold_addra <= x"00" & addr(3 downto 0);
threshold_addrb(8 downto 1) <= x"00";
threshold_addrb(0) <= htr_cntr_ra(0);
rst_state_LUT <= '0' when addr(14 downto 4) = sta_ctrl_addr(16 downto 6) else '1';
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(rst_state_LUT = '1')then
				state_LUT(3 downto 0) <= x"0";
			else
				state_LUT(3 downto 0) <= state;
			end if;
		end if;
  end if;
end process;
i_state: ram16XNs
    generic map( N => 4)
    port map(
        clk => clk,
        we => wr_state, 
        d => w_data(3 downto 0),
        a => addr(3 downto 0),
        o => state
        );			
wr_stateLUT <= '1' when we_byte(3) = '1' and addr(14 downto 4) = sta_ctrl_addr(16 downto 6) else '0';
wr_state <= strobe and wr_stateLUT;
state_LUT(31 downto 8) <= x"000000";
-- legal states defined as: "0001" overflow warning, "0010" out of sync, "0100" busy
--                          "1000" ready, "1100" error, "1111" disconnected
-- stateToGo_in(1 downto 0)
-- "01" out of sync
-- "10" error
-- "11" disconnected
i_stateConvert0: ROM16X1
   generic map(INIT => x"8004")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(0));
i_stateConvert1: ROM16X1
   generic map(INIT => x"9000")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(1));
-- stateToGo_in(2) '1' when overflow warning
i_stateConvert2: ROM16X1
   generic map(INIT => x"0002")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(2));
-- stateToGo_in(3)  '1' when busy
i_stateConvert3: ROM16X1
   generic map(INIT => x"0010")
	port map(     
      A0 => w_data(0),
      A1 => w_data(1),
      A2 => w_data(2),
      A3 => w_data(3),
      O  => stateToGo_in(3));
i_stateToGo: RAMB16_S4_S36
  port map(DIA => stateToGo_in,
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_stateLUT,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => threshold_addra,
					ADDRB => stateToGo_addrb,
					DOB => stateToGop
					);
i_merge_state: RAMB16_S2_S2
-- INIT_00 to INIt_1F of i_merge_state are all '0'
-- INIT_20 thru INIT_3F must be specified in dcc.ucf file
	  generic map(SIM_COLLISION_CHECK => "NONE",
      INIT_20 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_21 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_22 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_23 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_24 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_25 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_26 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_27 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_28 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_29 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2a => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_2b => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_2c => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2d => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_2e => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_2f => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_30 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_31 => x"ffffeaaae555e400e400e400e400e400ffffeaaae555e400e400e400e400e400",
      INIT_32 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e400e400e400e400e400",
      INIT_33 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_34 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_35 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_36 => x"ffffeaaae555e555e555e555e555e555ffffeaaae555e555e555e555e555e555",
      INIT_37 => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_38 => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_39 => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3a => x"ffffeaaaeaaaeaaaeaaaeaaaeaaaeaaaffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3b => x"ffffffffffffffffffffffffffffffffffffeaaaeaaaeaaaeaaaeaaaeaaaeaaa",
      INIT_3c => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3d => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3e => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff",
      INIT_3f => x"ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff")
  port map(DIA => "00",
  			  DIB => "00",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => '0',
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => merge_state_addr(25 downto 13),
					ADDRB => merge_state_addr(12 downto 0),
					DOA => merge_state(3 downto 2),
					DOB => merge_state(1 downto 0)
					);
process(clk)
begin
  if(clk'event and clk = '1')then
    merge_state_addr(12) <= wr_cntr(3);
    merge_state_addr(25) <= wr_cntr(3);
    for i in 0 to 3 loop
      merge_state_addr(2+i*3 downto i*3) <= gt_threshold(i) & stateToGo(1+i*4 downto i*4);
      merge_state_addr(15+i*3 downto 13+i*3) <= gt_threshold(i+4) & stateToGo(17+i*4 downto 16+i*4);
	 end loop;
  end if;
end process;
i_update_ow: delayN
  generic map(N=> 6) port map(d => rd_htr_status, ce => '1', c => clk, q => update_ow);
process(clk)
begin
  if(clk'event and clk = '1')then
    for i in 0 to 3 loop
			merge_ow(i) <= (stateToGo(2+i*8) and gt_threshold(i*2)) or (stateToGo(6+i*8) and gt_threshold(i*2+1));  
			merge_bz(i) <= (stateToGo(3+i*8) and gt_threshold(i*2)) or (stateToGo(7+i*8) and gt_threshold(i*2+1));  
		end loop;
		if(rd_htr_status = '1')then
			htr_num <= htr_cntr_ra(4 downto 1);
		end if;
		ow_tmp <= merge_ow(0) or merge_ow(1) or merge_ow(2) or merge_ow(3);
		bz_tmp <= merge_bz(0) or merge_bz(1) or merge_bz(2) or merge_bz(3);
		if(rst_cycle = '1')then
			ow <= (others => '0');
			bz <= (others => '0');
		elsif(update_ow = '1' and htr_num /= x"f")then
			ow(conv_integer(htr_num)) <= merge_ow(0) or merge_ow(1) or merge_ow(2) or merge_ow(3) or ow_tmp;
			bz(conv_integer(htr_num)) <= merge_bz(0) or merge_bz(1) or merge_bz(2) or merge_bz(3) or bz_tmp;
		end if;
		if(reset = '1' or rst_cycle = '1')then
			HTR_ow <= '0';
		elsif(ow(14 downto 0) /= "000000000000000")then
			HTR_ow <= '1';
		else
			HTR_ow <= '0';
		end if;	 
		if(reset = '1' or rst_cycle = '1' or en_HTR_bz = '0')then
			HTR_bz <= '0';
		elsif(bz(14 downto 0) /= "000000000000000")then
			HTR_bz <= '1';
		else
			HTR_bz <= '0';
		end if;
		if(reset = '1' or rst_cycle = '1')then
			HTR_state <= "00";
		else	
      HTR_state(1) <= merge_state(3) or merge_state(1);	 
      HTR_state(0) <= (merge_state(2) and (merge_state(3) or not merge_state(1))) or
						  (merge_state(0) and (merge_state(1) or not merge_state(3)));
    end if;						  
  end if;
end process;
end my_arch;
