library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity evt_bldr is
    port (
-- reset input, should be connected to the inverted LOCK output of dll
        reset: in STD_LOGIC;
        reset_err: in STD_LOGIC;
-- system clock input
        clk: in STD_LOGIC;
        version : in std_logic_vector(15 downto 0);
-- ttcrx event numbers
				ovfl_warning : out std_logic;
				evn_fifo_full : out std_logic;
				ttc_evcnt_reset : in std_logic;
				event_number_avl : in std_logic;
        event_number : in std_logic_vector(59 downto 0);
-- LRB data spying
				lrb_en : in std_logic_vector(14 downto 0);
-- download data write
        addr: in STD_LOGIC_VECTOR (15 downto 0);
        we_byte: in STD_LOGIC_VECTOR (3 downto 0);
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        run: in STD_LOGIC;
        n_strobe: in STD_LOGIC;
        err_state: out STD_LOGIC_VECTOR (1 downto 0);
        en_htr_ow: out STD_LOGIC;
        en_htr_bz: out STD_LOGIC;
-- counter inputs
        inc_l1ac : in std_logic;
        inc_bcnterr : in std_logic;
        inc_oc : in std_logic;
        inc_serr : in std_logic;
        inc_derr : in std_logic;
        inc_rate_OFW : in std_logic;
        inc_mon_cntr : in std_logic;
        inc_slink_wc : in std_logic;
        inc_slink_bc : in std_logic;
        inc_crc_err : in std_logic;
-- register and counter read back
        debug: out STD_LOGIC_VECTOR (31 downto 0);
        status_cntr: out STD_LOGIC_VECTOR (31 downto 0);
        evn_mm_cntr: out STD_LOGIC_VECTOR (31 downto 0);
        mmLUT_data: out STD_LOGIC_VECTOR (31 downto 0);
        evn_data: out STD_LOGIC_VECTOR (31 downto 0);
        evn_pntr: out STD_LOGIC_VECTOR (31 downto 0);
        counters: out STD_LOGIC_VECTOR (31 downto 0);
        timeout_data: out STD_LOGIC_VECTOR (31 downto 0);
        src_id_data: out STD_LOGIC_VECTOR (31 downto 0);
-- LED display signals
				htr_status : out std_logic_vector(14 downto 0);
				htr_error : out std_logic_vector(14 downto 0);
				htr_state : in std_logic_vector(1 downto 0);
        wp_addr: in STD_LOGIC_VECTOR (3 downto 0);
        dpo_wp: in STD_LOGIC_VECTOR (8 downto 0);
        rp_addr: out STD_LOGIC_VECTOR (3 downto 0);
        dpo_rp: out STD_LOGIC_VECTOR (8 downto 0);
        summary: in STD_LOGIC_VECTOR (31 downto 0);
        lrb_evn: in STD_LOGIC_VECTOR (40 downto 0);
        evn_ra: out STD_LOGIC_VECTOR (11 downto 0);
        smry_ra: out STD_LOGIC_VECTOR (11 downto 0);
-- number of HTR event block for the event being built
				htr_rd_cnt : out std_logic_vector(3 downto 0);
        htr_rd_cnt_str: out STD_LOGIC;
-- request one HTR event block from LRBs thru edc
-- bit(11 downto 8) lrb_id
-- bit(7 downto 0) buffer number
        htr_buf_addr: out STD_LOGIC_VECTOR (11 downto 0);
-- htr_buf_length is 32-bit word counts
        htr_buf_length: out STD_LOGIC_VECTOR (8 downto 0);
        htr_buf_req: out STD_LOGIC;
-- HTR data to be added to event stream
        htr_buf_data: in STD_LOGIC_VECTOR (32 downto 0);
        htr_buf_val: in STD_LOGIC;
--        htr_buf_full: out STD_LOGIC_VECTOR (14 downto 0);
-- TTCrx states
				TTS : IN std_logic_vector(3 downto 0);
-- data in write enable
        slink_we: out STD_LOGIC;
-- slink_data(35) is Calib_trig flag
-- slink_data(34) is slink_ctrl, slink_data(33) is header, slink_data(32) is trailer
        slink_data: out STD_LOGIC_VECTOR (35 downto 0)
    );
end evt_bldr;

architecture my_arch of evt_bldr is
signal strobe: std_logic := '0';
signal slink_we_i: std_logic := '0';
signal event_data: std_logic_vector(34 downto 0) := (others => '0');
signal event_data_dl: std_logic_vector(15 downto 0) := (others => '0');
signal d_crc_data: std_logic_vector(31 downto 0) := (others => '0');
signal crc_data: std_logic_vector(31 downto 0) := (others => '0');
signal crc_data_a: std_logic_vector(1 downto 0) := (others => '0');
signal crc: std_logic_vector(15 downto 0) := (others => '0');
signal calib_trig: std_logic := '0';
signal init_crc: std_logic := '0';
signal sel_crc: std_logic := '0';
signal first_word: std_logic := '0';
signal high_word: std_logic := '0';
signal wr_high: std_logic := '0';
signal wr_low: std_logic := '0';
signal we_crc: std_logic := '0';
signal rst_cycle: std_logic := '0';
signal rst_cycle_dl: std_logic := '0';
signal rst_done: std_logic := '0';
signal we_rp: std_logic := '0';
signal rp_plus_one: std_logic_vector(8 downto 0) := (others => '0');
signal rp: std_logic_vector(8 downto 0) := (others => '0');
signal dpo_rpp: std_logic_vector(8 downto 0) := (others => '0');
signal rp_addr_i: std_logic_vector(3 downto 0) := (others => '0');
signal last_lrb: std_logic := '0';
signal htr_en: std_logic_vector(14 downto 0) := (others => '0');
signal htr_buf_cnt: std_logic_vector(8 downto 0) := (others => '0');
signal htr_buf_req_i: std_logic := '0';
signal sel_header: std_logic := '0';
signal ec_header_fifo_wa_h: std_logic := '0';
signal odd_htr_length: std_logic := '0';
signal wr_padding: std_logic := '0';
signal start_header: std_logic := '0';
signal oc_match: std_logic := '0';
signal bc_match: std_logic := '0';
signal evn_match: std_logic := '0';
signal ec_bc_evn_match: std_logic := '0';
signal bc_evn_match: std_logic_vector(3 downto 0) := (others => '0');
signal lrb_smry: std_logic_vector(31 downto 0) := (others => '0');
signal htr_header: std_logic_vector(34 downto 0) := (others => '0');
signal header: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header1: std_logic_vector(35 downto 0) := (others => '0');
signal daq_header2: std_logic_vector(35 downto 0) := (others => '0');
signal dcc_header: std_logic_vector(35 downto 0) := (others => '0');
signal wr_summary: std_logic := '0';
signal EPV: std_logic_vector(14 downto 0) := (others => '0');
signal daq_trailer: std_logic_vector(34 downto 0) := (others => '0');
signal evt_length: std_logic_vector(12 downto 0) := (others => '0');
signal header_in: std_logic_vector(35 downto 0) := (others => '0');
signal header_out: std_logic_vector(35 downto 0) := (others => '0');
signal header_fifo_wa: std_logic_vector(8 downto 0) := (others => '0');
signal header_fifo_ra: std_logic_vector(8 downto 0) := (others => '0');
constant twentyfour: std_logic_vector(12 downto 0) := "0000000011000";
signal trailer_sent: std_logic := '0';
signal wr_header: std_logic := '0';
signal ec_header_fifo_ra: std_logic := '0';
signal header_sent: std_logic := '0';
signal htr_rd_cnt_i: std_logic_vector(4 downto 0) := (others => '0');
--signal htr_sent: std_logic := '0';
signal chk_htr_rd_cnt: std_logic := '0';
signal htr_req_cnt: std_logic_vector(3 downto 0) := (others => '0');
signal htr_req_cnt_que: std_logic_vector(4 downto 0) := (others => '0');
signal start_trailer: std_logic := '0';
signal fake_lrb: std_logic := '0';
signal wr_htr: std_logic := '0';
signal wr_tail: std_logic := '0';
signal wr_tail_dl: std_logic := '0';
signal sel_head: std_logic := '0';
signal wr_head: std_logic := '0';
signal ec_rp_addr: std_logic := '0';
signal clear: std_logic := '0';
signal check_next: std_logic := '0';
signal skip: std_logic := '0';
signal missing: std_logic := '0';
signal start_read: std_logic := '0';
signal evn_seq: std_logic := '0';
signal we_last_evn: std_logic := '0';
signal last_evn: std_logic_vector(3 downto 0) := (others => '0');
signal evn_diff: std_logic_vector(23 downto 0) := (others => '0');
signal lrb_match: std_logic_vector(11 downto 0) := (others => '0');
signal lrb_match_q: std_logic_vector(2 downto 0) := (others => '0');
signal matching: std_logic_vector(3 downto 0) := (others => '0');
signal en_scan: std_logic := '0';
signal htr_scan: std_logic := '0';
signal stop_htr_scan: std_logic := '0';
signal two_htr_buf: std_logic := '0';
signal one_evn: std_logic := '0';
signal htr_time_out: std_logic := '0';
signal ld_evn_wait: std_logic := '0';
signal evn_timeout: std_logic := '0';
signal evn_wait_cntr: std_logic_vector(10 downto 0) := (others => '0');
signal ld_htr_wait: std_logic := '0';
signal htr_wait_ovfl: std_logic := '0';
signal htr_wait_cntr: std_logic_vector(15 downto 0) := (others => '0');
signal d_to_cnt: std_logic_vector(1 downto 0) := (others => '0');
signal to_cnt: std_logic_vector(1 downto 0) := (others => '0');
signal new_rp: std_logic := '0';
signal new_rp_dl: std_logic := '0';
signal htr_rd_length: std_logic_vector(9 downto 0) := (others => '0');
signal evn_avl: std_logic := '0';
signal wr_evn: std_logic := '0';
signal ovfl_warning_i : std_logic := '0';
signal evn_fifo_wa: std_logic_vector(9 downto 0) := (others => '0');
signal evn_fifo_rap: std_logic_vector(9 downto 0) := (others => '0');
signal evn_fifo_ra: std_logic_vector(9 downto 0) := (others => '0');
signal evn_wc: std_logic_vector(7 downto 0) := (others => '0');
signal evcnt: std_logic_vector(23 downto 0) := x"000001";
signal evn_in: std_logic_vector(89 downto 0) := (others => '0');
signal evn_out: std_logic_vector(89 downto 0) := (others => '0');
signal evt_ty: std_logic_vector(3 downto 0) := (others => '0');
signal cal_ty: std_logic_vector(3 downto 0) := (others => '0');
signal source_id: std_logic_vector(31 downto 0) := (others => '0');
signal timeout: std_logic_vector(31 downto 0) := x"00000000";
signal NoL1ac: std_logic := '0';
signal NoWaitNext: std_logic := '0';
signal NoSkip: std_logic := '0';
signal NoMiss: std_logic := '0';
signal EnFakeL1ac: std_logic := '0';
signal EnHTRtimeout: std_logic := '0';
signal EvN_mm: std_logic := '0';
signal BcN_mm: std_logic := '0';
signal CT_EvN_mm: std_logic := '0';
signal CT_BcN_mm: std_logic := '0';
signal L1_EvN_mm: std_logic := '0';
signal L1_BcN_mm: std_logic := '0';
signal en_wr_cntr_que_a: std_logic := '0';
signal DCC_err: std_logic_vector(11 downto 0) := (others => '0');
-- 32 bit counters
signal evt_bldr_bc: std_logic_vector(1 downto 0) := (others => '0');
signal crc_err_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal slink_bc: std_logic_vector(1 downto 0) := (others => '0');
signal mon_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal l1ac_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal CT_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal CT_EvN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal CT_BcN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal L1_EvN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal L1_BcN_mmc: std_logic_vector(1 downto 0) := (others => '0');
signal bcnterr_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal rate_OFW_cntr: std_logic_vector(1 downto 0) := (others => '0');
signal ttc_state: std_logic_vector(1 downto 0) := (others => '0');
signal ld_evt_bldr_bc: std_logic := '0';
signal ld_slink_bc: std_logic := '0';
signal ld_crc_err_cntr: std_logic := '0';
signal ld_mon_cntr: std_logic := '0';
signal ld_l1ac_cntr: std_logic := '0';
signal ld_CT_cntr: std_logic := '0';
signal ld_CT_EvN_mmc: std_logic := '0';
signal ld_CT_BcN_mmc: std_logic := '0';
signal ld_L1_EvN_mmc: std_logic := '0';
signal ld_L1_BcN_mmc: std_logic := '0';
signal ld_bcnterr_cntr: std_logic := '0';
signal ld_rate_OFW_cntr: std_logic := '0';
signal rst_counterb: std_logic := '0';
signal counterb_inc: std_logic_vector(31 downto 0) := (others => '0');
signal counterb_addr: std_logic_vector(3 downto 0) := (others => '0');
signal counterb_in: std_logic_vector(31 downto 0) := (others => '0');
signal counterb_out: std_logic_vector(31 downto 0) := (others => '0');
signal counterb: std_logic_vector(31 downto 0) := (others => '0');
-- 64 bit counters
signal evt_bldr_wc: std_logic_vector(7 downto 0) := (others => '0');
signal slink_wc: std_logic_vector(6 downto 0) := (others => '0');
signal serr_cntr: std_logic_vector(5 downto 0) := (others => '0');
signal derr_cntr: std_logic_vector(5 downto 0) := (others => '0');
signal ready_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal busy_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal ovfl_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal sync_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal run_cntr: std_logic_vector(6 downto 0) := (others => '0');
signal ld_evt_bldr_wc: std_logic := '0';
signal ld_slink_wc: std_logic := '0';
signal ld_serr_cntr: std_logic := '0';
signal ld_derr_cntr: std_logic := '0';
signal ld_ready_cntr: std_logic := '0';
signal ld_busy_cntr: std_logic := '0';
signal ld_ovfl_cntr: std_logic := '0';
signal ld_sync_cntr: std_logic := '0';
signal ld_run_cntr: std_logic := '0';
signal mux2_a: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_b: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_c: std_logic_vector(7 downto 0) := (others => '0');
signal mux2_d: std_logic_vector(7 downto 0) := (others => '0');
signal mux4_a: std_logic_vector(7 downto 0) := (others => '0');
signal mux4_b: std_logic_vector(7 downto 0) := (others => '0');
signal scan_cntr_que_a: std_logic := '0';
signal scan_cntr_que_a_dl: std_logic := '0';
signal scan_cntr_que_a_dl2: std_logic := '0';
signal wr_cntr_que_a: std_logic := '0';
signal rd_cntr_que_a: std_logic := '0';
signal dav_cntr_que_a: std_logic := '0';
signal sel_cntr_que_a: std_logic := '0';
signal sel_cntr_que_b: std_logic := '0';
signal cntr_que_a_in: std_logic_vector(11 downto 0) := (others => '0');
signal cntr_que_a: std_logic_vector(11 downto 0) := (others => '0');
signal scan_cntr: std_logic_vector(3 downto 0) := (others => '0');
signal scan_cntr_a: std_logic_vector(3 downto 0) := (others => '0');
signal scan_cntr_b: std_logic_vector(3 downto 0) := (others => '0');
signal wr_cntr_que_b: std_logic := '0';
signal rd_cntr_que_b: std_logic := '0';
signal dav_cntr_que_b: std_logic := '0';
signal counter_cycle: std_logic_vector(1 downto 0) := (others => '0');
signal wr_counter: std_logic := '0';
signal rst_counter_n: std_logic := '0';
--signal en_counter_out: std_logic := '0';
signal rd_counter_high: std_logic := '0';
signal addr_q: std_logic_vector(1 downto 0) := (others => '0');
signal cntr_que_b_in: std_logic_vector(8 downto 0) := (others => '0');
signal cntr_que_b: std_logic_vector(8 downto 0) := (others => '0');
signal d_counter_wap: std_logic_vector(6 downto 0) := (others => '0');
signal counter_wap: std_logic_vector(6 downto 0) := (others => '0');
signal counter_wa: std_logic_vector(8 downto 0) := (others => '0');
signal counter_ra: std_logic_vector(8 downto 0) := (others => '0');
signal increment: std_logic_vector(63 downto 0) := (others => '0');
signal counter_in: std_logic_vector(63 downto 0) := (others => '0');
signal counter_out: std_logic_vector(63 downto 0) := (others => '0');
signal counter_rb: std_logic_vector(63 downto 0) := (others => '0');
signal counter_high: std_logic_vector(31 downto 0) := (others => '0');
signal old_cntr_addr: std_logic_vector(14 downto 0) := (others => '0');
signal evn_mm_cntr_in : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_out : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_out_q : std_logic_vector(31 downto 0) := (others => '0');
signal evn_mm_cntr_aa : std_logic_vector(8 downto 0) := (others => '0');
signal evn_mm_cntr_ab : std_logic_vector(8 downto 0) := (others => '0');
signal evn_mm_cntr_ab_dl : std_logic_vector(6 downto 0) := (others => '0');
signal evn_mm_cntr_a : std_logic_vector(8 downto 0) := (others => '0');
signal wr_evn_mm_cntr : std_logic_vector(5 downto 0) := (others => '0');
signal rst_mm_cntr : std_logic := '0';
signal mmLUT : std_logic_vector(7 downto 0) := (others => '0');
signal mmLUT_spo : std_logic_vector(7 downto 0) := (others => '0');
signal we_mmLUT: std_logic := '0';
signal mmLUT_scan : std_logic_vector(3 downto 0) := (others => '0');
signal CT_err_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal L1_err_cntr : std_logic_vector(7 downto 0) := (others => '0');
signal err_cntr : std_logic_vector(3 downto 0) := (others => '0');
signal threshold : std_logic_vector(3 downto 0) := (others => '0');
signal stateToGo : std_logic_vector(1 downto 0) := (others => '0');
signal bcnt_err: std_logic := '0';
signal bcnt_err_cntr : std_logic_vector(3 downto 0) := (others => '0');
signal faked_evn: std_logic := '0';
signal saved_TTS : std_logic_vector(3 downto 0) := (others => '0');
------------------------------------------------------
-- debugging
------------------------------------------------------
begin
ovfl_warning <= ovfl_warning_i;
htr_buf_req <= htr_buf_req_i;
rp_addr <= rp_addr_i;
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
  end if;
end process;
---- buffer pointers
process(clk)
begin
  if(clk'event and clk = '1')then
		if(rst_cycle = '1')then
			rp_plus_one <= (others => '0');
		elsif(strobe = '1')then
			rp_plus_one <= rp + 1;
		end if;
		dpo_rp <= dpo_rpp;
  end if;
end process;
-- read pointer
i_rp: ram16XNd
    generic map( N => 9)
    port map(
        clk => clk,
        we => we_rp, 
        d => rp_plus_one,
        a => rp_addr_i,
        dpra => wp_addr,
        spo => rp,
        dpo => dpo_rpp);
we_rp <= strobe and (rst_cycle or skip or htr_buf_req_i);
htr_buf_cnt <= dpo_wp - rp;
-- reset starts a cycle to clear all pointers
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			rst_cycle <= '1';
			rst_cycle_dl <= '0';
		elsif(strobe = '1')then
			rst_cycle <= rst_cycle and not last_lrb;
			rst_cycle_dl <= rst_cycle;
    end if;
		if(reset = '1')then
			rp_addr_i <= (others => '0');
		elsif(ec_rp_addr = '1')then
			if(rp_addr_i(3 downto 1) = "111")then
				rp_addr_i <= (others => '0');
			else
				rp_addr_i <= rp_addr_i + 1;
			end if;
		end if;
  end if;
end process;
-- read pointer address
last_lrb <= '1' when rp_addr_i(3 downto 1) = "111" else '0';
ec_rp_addr <= strobe and (rst_cycle or htr_buf_req_i or fake_lrb or missing);
-- slink output control
-- to include calculated crc into daq trailer
i_slink_we: delayN generic map(N => 6) port map(d => slink_we_i, ce => '1', c => clk, q => slink_we);
--i_calib_trig: delayN generic map(N => 6) port map(d => calib_trig, ce => '1', c => clk, q => slink_data(35));
slink_data(35) <= '0';
g_slink_data_h: for i in 0 to 2 generate
	i_slink_data_h: delayN generic map(N => 6) port map(d => event_data(32+i), ce => '1', c => clk, q => slink_data(32+i));
end generate;
g_slink_data_l: for i in 0 to 15 generate
	i_slink_data_l: delayN generic map(N => 6) port map(d => event_data(i), ce => '1', c => clk, q => slink_data(i));
end generate;
g_event_data_dl: for i in 0 to 15 generate
	i_event_data_dl: delayN generic map(N => 5) port map(d => event_data(16+i), ce => '1', c => clk, q => event_data_dl(i));
end generate;
i_sel_crc: delayN generic map(N => 7) port map(d => start_trailer, ce => '1', c => clk, q => sel_crc);
-- this is used to swap the order of the high and low 32-bit words
g_crc_data: for i in d_crc_data'range generate
  i_srle: srl16e
		port map(
			 d => event_data(i),
			 ce => slink_we_i,
			 clk => clk,
			 a0 => crc_data_a(0),
			 a1 => crc_data_a(1),
			 a2 => '0',
			 a3 => '0',
			 q  => d_crc_data(i));
end generate;
i_init_crc: delayN generic map(N => 3) port map(d => first_word, ce => '1', c => clk, q => init_crc);
process(clk)
begin
  if(clk'event and clk = '1')then
    crc_data <= d_crc_data;
		first_word <= sel_head and not wr_head;
		high_word <= (high_word xor slink_we_i) and not first_word;
    crc_data_a(1) <= slink_we_i and not high_word;
    crc_data_a(0) <= not slink_we_i and not high_word;
-- two clocks after high word written to swapper, writes both words to crc16D32
		wr_high <= high_word and slink_we_i;
		wr_low <= wr_high;
		we_crc <= wr_high or wr_low;
    if(sel_crc = '1')then
      slink_data(31 downto 16) <= crc;
		else
      slink_data(31 downto 16) <= event_data_dl;
		end if;
  end if;
end process;
i_crc: crc16D32
	port map(
		clk => clk,
		init_crc => init_crc,
		we_crc => we_crc,
		d => crc_data,
		crc => crc);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			start_trailer <= '0';
			wr_tail_dl <= '0';
			odd_htr_length <= '0';
			wr_padding <= '0';
			sel_head <= '0';
			wr_head <= '0';
			wr_htr <= '0';
			wr_tail <= '0';
			slink_we_i <= '0';
			evt_length <= (others => '0');
		else
			start_trailer <= chk_htr_rd_cnt and not htr_rd_cnt_i(4);
			wr_tail_dl <= wr_tail;
-- add padding if htr data are not 64-bit aligned
--			wr_padding <= chk_htr_rd_cnt and not htr_rd_cnt_i(4) and not(evt_length(0) xor wr_htr xor slink_we_i);
			odd_htr_length <= not wr_head and (odd_htr_length xor wr_htr);
			wr_padding <= chk_htr_rd_cnt and not htr_rd_cnt_i(4) and not odd_htr_length;
			sel_head <= ec_header_fifo_ra;
			wr_head <= sel_head;
			wr_htr <= htr_buf_val;
			wr_tail <= start_trailer or (wr_tail and not wr_tail_dl);
			slink_we_i <= wr_htr or wr_padding or wr_head or wr_tail;
			if(slink_we_i = '1')then
				if(header_sent = '1')then
					evt_length <= twentyfour;
				else
					evt_length <= evt_length + 1;
				end if;
			end if;
    end if;
  end if;
end process;
-- output multiplexer
process(clk)
begin
  if(clk'event and clk = '1')then
    if(wr_padding = '1')then
-- fill with zeros
			event_data <= (others => '0');
    elsif(wr_tail = '1')then
-- daq trailer
			event_data <= daq_trailer;
		else
-- daq header, dcc header, htr summary and htr data
			event_data <= htr_header;
    end if;
    if(sel_head = '1')then
-- daq header, dcc header and htr header
			htr_header <= header_out(34 downto 0);
		elsif(htr_buf_val = '1')then
-- htr data
			htr_header <= "000" & htr_buf_data(31 downto 0);
    end if;
-- check calib_trig
		calib_trig <= header_out(35) and first_word;
    if(wr_tail = '1')then
-- daq trailer high word
			daq_trailer <= "101" & x"a0000" & evt_length(12 downto 1);
		else
-- daq trailer low word	with crc forced to zero
			daq_trailer <= "100" & x"0000" & source_id(19 downto 12) & TTS & x"0";
    end if;
-- DAQ header, DCC header and HTR summary
    if(strobe = '1')then
			if(sel_header = '1')then
				header_in <= header;
			elsif(fake_lrb = '1' or missing = '1')then
        header_in <= x"00000" & lrb_smry(15) & "000" & x"000";
			else
        header_in <= x"0" & lrb_smry(31 downto 14) & (bc_evn_match(2) and bc_evn_match(1)) & bc_evn_match(0) & lrb_smry(11 downto 0);
      end if;
			if(header_fifo_wa(2 downto 1) = "01")then
        header <= dcc_header;
-- high word here
        dcc_header <= x"000" & version(7 downto 0) & x"0" & DCC_err;
			else
				header <= daq_header;
-- low word here
        dcc_header <= "0000000" & EPV & "00" & saved_TTS & x"02";
      end if;
			if(sel_header = '1')then
        daq_header <= daq_header2;
			else
				daq_header <= daq_header1;
      end if;
			saved_TTS <= evn_out(79 downto 76);
		end if;
  end if;
end process;
evt_ty <= "00" & evn_out(68) & not evn_out(68);
cal_ty <= evn_out(75 downto 72);
lrb_smry <= summary(31 downto 16) & htr_en(0) & summary(14 downto 0); 
daq_header1 <= x"45" & evt_ty & evn_out(23 downto 0) when start_header = '1' else
							 evn_out(68) & "110" & evn_out(35 downto 24) & source_id(11 downto 0) & x"08";
--              high word										  low word
daq_header2 <= x"4" & evn_out(63 downto 36) & x"0" when wr_header = '0' else x"40" & cal_ty & x"00000" & evn_out(67 downto 64);
i_header_fifo: RAMB16_S36_S36
	generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => header_in(31 downto 0),
  			  DIPA => header_in(35 downto 32),
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => strobe,
  			  ENB => '1',
  			  WEA => wr_header,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => header_fifo_wa,
					ADDRB => header_fifo_ra,
					DOB => header_out(31 downto 0),
					DOPB => header_out(35 downto 32)
			  );
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			header_fifo_wa(8 downto 5) <= "0000";
		elsif(strobe = '1')then
			header_fifo_wa(6 downto 5) <= header_fifo_wa(6 downto 5) + ec_header_fifo_wa_h;
		end if;
		if(strobe = '1')then
			if(sel_header = '1')then
				header_fifo_wa(4 downto 0) <= header_fifo_wa(4 downto 0) + 1;
			elsif(start_header = '1')then
				header_fifo_wa(4 downto 0) <= "11111";
			else
				header_fifo_wa(4 downto 0) <= ('0' & rp_addr_i) + "00110";
			end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			wr_summary <= start_read or fake_lrb or missing;
			if(wr_summary = '1')then
				EPV(14) <= header_in(15) and header_in(14) and header_in(13) and header_in(12) and not header_in(11);
				EPV(13 downto 0) <= EPV(14 downto 1);
			end if;
			if(ec_header_fifo_wa_h = '1')then
				htr_status <= EPV;
				htr_error <= not EPV and lrb_en;
				EvN_mm <= '0';
				BcN_mm <= '0';
				CT_EvN_mm <= evn_out(68) and EvN_mm;
				CT_BcN_mm <= evn_out(68) and BcN_mm;
				L1_EvN_mm <= not evn_out(68) and EvN_mm;
				L1_BcN_mm <= not evn_out(68) and BcN_mm;
			else
				htr_status <= (others => '0');
				htr_error <= (others => '0');
				if(wr_summary = '1' and header_in(15) = '1' and (header_in(14) = '0' or header_in(12) = '0'))then
					EvN_mm <= '1';
				end if;
				if(wr_summary = '1' and header_in(15) = '1' and header_in(14 downto 13) /= "11")then
					BcN_mm <= '1';
				end if;
				CT_EvN_mm <= '0';
				CT_BcN_mm <= '0';
				L1_EvN_mm <= '0';
				L1_BcN_mm <= '0';
			end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			sel_header <= '0';
			wr_header <= '0';
			ec_header_fifo_wa_h <= '0';
		elsif(strobe = '1')then
			sel_header <= start_header or (sel_header and not (not header_fifo_wa(3) and header_fifo_wa(2)));
			wr_header <= sel_header or start_read or fake_lrb or missing;
			ec_header_fifo_wa_h <= sel_header and not header_fifo_wa(3) and header_fifo_wa(2);
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			start_header <= '0';
			fake_lrb <= '0';
		elsif(strobe = '1')then
			start_header <= stop_htr_scan;
			fake_lrb <= not fake_lrb and not new_rp and not htr_en(0) and ((en_scan and not last_lrb) or htr_scan);
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			header_fifo_ra(8 downto 5) <= "0000";
		elsif(header_sent = '1')then
			header_fifo_ra(6 downto 5) <= header_fifo_ra(6 downto 5) + 1;
    end if;
    if(reset = '1')then
			header_fifo_ra(4 downto 0) <= "00000";
		elsif(ec_header_fifo_ra = '1')then
			if(header_sent = '1')then
				header_fifo_ra(4 downto 0) <= "00000";
			else
				header_fifo_ra(4 downto 0) <= header_fifo_ra(4 downto 0) + 1;
			end if;
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			trailer_sent <= '1';
			ec_header_fifo_ra <= '0';
			header_sent <= '0';
		else
			trailer_sent <= start_trailer or (trailer_sent and not ec_header_fifo_ra);
			ec_header_fifo_ra <= (trailer_sent and htr_rd_cnt_i(4)) or (ec_header_fifo_ra and not header_sent);
			header_sent <= header_fifo_ra(4) and header_fifo_ra(2) and header_fifo_ra(1) and not header_fifo_ra(0);
    end if;
  end if;
end process;
-- LRB enable
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(en_scan = '0' and htr_scan = '0')then
				htr_en <= lrb_en;
			elsif(ec_rp_addr = '1')then
				htr_en <= htr_en(0) & htr_en(14 downto 1);
			end if;
    end if;
  end if;
end process;
oc_match <= '1' when evn_out(40 downto 36) = lrb_evn(40 downto 36) else '0';
bc_match <= '1' when evn_out(35 downto 24) = lrb_evn(35 downto 24) else '0';
evn_match <= '1' when evn_out(23 downto 0) = lrb_evn(23 downto 0) else '0';
process(evn_out, lrb_evn)
begin
	evn_diff <= evn_out(23 downto 0) - lrb_evn(23 downto 0);
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1' and ec_bc_evn_match = '1')then
			bc_evn_match <= evn_out(68) & oc_match & bc_match & evn_match;
    end if;
		if(strobe = '1')then
			ec_bc_evn_match <= not two_htr_buf or new_rp or en_scan;
		end if;
  end if;
end process;
-- HTR buffer readout control
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			htr_buf_req_i <= '0';
		elsif(strobe = '1')then
			htr_buf_req_i <= start_read;
    end if;
		if(strobe = '1')then
			htr_rd_length <= summary(9 downto 0) - 1;
			htr_buf_addr <= rp_addr_i & rp(7 downto 0);
    end if;
		if(reset = '1')then
			htr_req_cnt <= (others => '0');
		elsif(strobe = '1')then
			if(ec_header_fifo_wa_h = '1')then
				htr_req_cnt <= (others => '0');
			elsif(htr_buf_req_i = '1')then
				htr_req_cnt <= htr_req_cnt + 1;
			end if;
		end if;
		if(reset = '1')then
			htr_req_cnt_que <= (others => '0');
		elsif(ec_header_fifo_wa_h = '1' and strobe = '1')then
			htr_req_cnt_que <= '1' & htr_req_cnt;
		elsif(htr_rd_cnt_i(4) = '0')then
			htr_req_cnt_que(4) <= '0';
		end if;
		if(reset = '1')then
			htr_rd_cnt_i <= (others => '0');
		elsif(htr_req_cnt_que(4) = '1' and htr_rd_cnt_i(4) = '0')then
			htr_rd_cnt_i <= htr_req_cnt_que;
		elsif((htr_buf_val = '1' and htr_buf_data(32) = '1') or header_sent = '1')then
			htr_rd_cnt_i <= htr_rd_cnt_i - 1;
		end if;
    if(reset = '1')then
			chk_htr_rd_cnt <= '0';
		else
			chk_htr_rd_cnt <= (htr_buf_val and htr_buf_data(32)) or header_sent;
    end if;
		htr_rd_cnt <= htr_rd_cnt_i(3 downto 0) - 1;
		if(htr_rd_cnt_i(3 downto 0) /= x"0")then
			htr_rd_cnt_str <= header_sent;
		else
			htr_rd_cnt_str <= '0';
		end if;
  end if;
end process;
htr_buf_length <= htr_rd_length(8 downto 0);
-- event number input and timeout
evn_timeout <= evn_wait_cntr(10);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(ttc_evcnt_reset = '1' or reset = '1')then
				evcnt <= x"000001";
			elsif(wr_evn = '1')then
				evcnt <= evcnt + 1;
			end if;
		end if;
  end if;
end process;
evn_in(89 downto 24) <= "10" & x"0000000000000000" when evn_timeout = '1' else "000000" & event_number;
evn_in(23 downto 0) <= evcnt;
-- TTCrx event number fifo
g_evn_fifo: for i in 0 to 4 generate
  i_evn_fifo: RAMB16_S18_S18
	  generic map(SIM_COLLISION_CHECK => "NONE")
    port map(DIA => evn_in(i*18+15 downto i*18),
				DIPA => evn_in(i*18+17 downto i*18+16),
				DIB => x"0000",
				DIPB => "00",
				ENA => strobe,
				ENB => strobe,
				WEA => wr_evn,
				WEB => '0',
				SSRA => '0',
				SSRB => '0',
			  CLKA => clk,
			  CLKB => clk,
			  ADDRA => evn_fifo_wa,
			  ADDRB => evn_fifo_rap,
			  DOPB => evn_out(i*18+17 downto i*18+16),
			  DOB => evn_out(i*18+15 downto i*18)
			  );
end generate;
evn_fifo_rap <= evn_fifo_ra when run = '1' else addr(11 downto 2);
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(reset = '1')then
				evn_fifo_wa <= (others => '0');
				evn_fifo_ra <= (others => '0');
				evn_wc <= (others => '0');
			else
				if(wr_evn = '1')then
					evn_fifo_wa <= evn_fifo_wa - 1;
				end if;
				if(ec_header_fifo_wa_h = '1')then
					evn_fifo_ra <= evn_fifo_ra - 1;
				end if;
				if(wr_evn /= ec_header_fifo_wa_h)then
					if(wr_evn = '1')then
						evn_wc <= evn_wc + 1;
					else
						evn_wc <= evn_wc - 1;
					end if;
				end if;
			end if;			
		end if;
  end if;
end process;
evn_avl <= '0' when evn_wc = x"00" else '1';
one_evn <= '1' when evn_wc = x"01" else '0';
wr_evn <= evn_timeout when NoL1ac = '1' else event_number_avl;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			new_rp <= '1';
			new_rp_dl <= '0';
			ovfl_warning_i <= '0';
			evn_fifo_full <= '0';
			en_htr_ow <= '0';
			en_htr_bz <= '0';
		elsif(strobe = '1')then
			new_rp <= fake_lrb or missing or skip or htr_buf_req_i;
			new_rp_dl <= new_rp;
-- when reached 0x60, throttle lv1. Return only after go below 0x40
			if(evn_wc(7 downto 6) = "00")then
				ovfl_warning_i <= '0';
      elsif(evn_wc(5) = '1')then
				ovfl_warning_i <= '1';
			end if;
			if(evn_wc(7 downto 4) = x"f")then
				evn_fifo_full <= '1';
			else
				evn_fifo_full <= '0';
			end if;
			if(evn_wc(4) = '1' and evn_wc(1) = '1')then
				en_htr_ow <= '1';
			elsif(evn_wc(7 downto 5) = "000" and evn_wc(2 downto 0) = "000")then
				en_htr_ow <= '0';
			end if;
			if(evn_wc(5) = '1' and evn_wc(3 downto 2) = "11")then
				en_htr_bz <= '1';
			elsif(evn_wc(7 downto 5) = "000")then
				en_htr_bz <= '0';
			end if;
    end if;
  end if;
end process;
two_htr_buf <= '0' when htr_buf_cnt(8 downto 1) = x"00" else '1';
i_matching: RAMB16_S4
     generic map(
-- algorithm
      INIT_00 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_01 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_02 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_03 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_04 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_05 => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_06 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_07 => x"8880808080808080882010808080808088808080808080808820108080808080",
      INIT_08 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_09 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_0a => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0b => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_0d => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_0e => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_0f => x"8880808080808080888010808080808088808080808080808880108080808080",
      INIT_10 => x"1188118811221188118811881122118820202020202020202020202020202020",
      INIT_11 => x"1188118811221188118811881122118888444444884444448844444488444444",
      INIT_12 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_13 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_14 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_15 => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_16 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_17 => x"8880808088808080882010808820108088808080888080808820108088201080",
      INIT_18 => x"1188118811881188118811881188118820202020202020202020202020202020",
      INIT_19 => x"1188118811881188118811881188118888444444884444448844444488444444",
      INIT_1a => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1b => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_1d => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_1e => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_1f => x"8880808088808080888010808880108088808080888080808880108088801080",
      INIT_20 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_21 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_22 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_23 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_24 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_25 => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_26 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_27 => x"8888888888888888882211888888888888888888888888888822118888888888",
      INIT_28 => x"1188118888888888118811888888888820202020202020202020202020202020",
      INIT_29 => x"1188118888888888118811888888888888444444444444448844444444444444",
      INIT_2a => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2b => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_2d => x"8888888888888888888888888888888888444444444444448844444444444444",
      INIT_2e => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_2f => x"8888888888888888888811888888888888888888888888888888118888888888",
      INIT_30 => x"1188118811221188118811881122118820202020202020202020202020202020",
      INIT_31 => x"1188118811221188118811881122118888444444884444448844444488444444",
      INIT_32 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_33 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_34 => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_35 => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_36 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_37 => x"8888888888888888882211888822118888888888888888888822118888221188",
      INIT_38 => x"1188118811881188118811881188118820202020202020202020202020202020",
      INIT_39 => x"1188118811881188118811881188118888444444884444448844444488444444",
      INIT_3a => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3b => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3c => x"8888888888888888888888888888888820202020202020202020202020202020",
      INIT_3d => x"8888888888888888888888888888888888444444884444448844444488444444",
      INIT_3e => x"8888888888888888888811888888118888888888888888888888118888881188",
      INIT_3f => x"8888888888888888888811888888118888888888888888888888118888881188")
  port map(DI => x"0",
				EN => strobe,
				WE => '0',
				SSR => clear,
			  CLK => clk,
			  ADDR => lrb_match,
			  DO => matching(3 downto 0)
			  );
clear <= reset or not htr_scan or htr_buf_req_i or new_rp or matching(3) or matching(1) or matching(0);
-- if NoWaitNext set, do not wait for next HTR event
lrb_match(11) <= NoWaitNext when check_next = '0' else lrb_match_q(2); 
lrb_match(10) <= evn_seq; 
-- if NoMiss set, do not insert HTR event unless timeout
lrb_match(9) <= NoMiss; 
-- if EnHTRtimeout bit is not set, DCC waits for HTR event forever
--lrb_match(8) <= lrb_match_q(1);
lrb_match(8) <= '0';-- xor evn_out(68); 
lrb_match(7) <= htr_buf_cnt(0) and not two_htr_buf;  
-- when only one HTR event, bit 6 and 5 can be used to input extra info
lrb_match(6) <= two_htr_buf when lrb_match(7) = '0' else lrb_match_q(1); 
--lrb_match(5) <= check_next when lrb_match(7) = '0' else lrb_match_q(0); 
lrb_match(5) <= check_next; 
lrb_match(4) <= one_evn; 
lrb_match(3) <= '1' when evn_diff(4 downto 0) = "11111" or evn_diff(4 downto 0) = "00001" or evn_diff = x"000000" else '0'; 
lrb_match(2) <= '1' when evn_diff(23 downto 4) = x"fffff" or evn_diff(3 downto 0) = "0000" else '0'; 
lrb_match(1) <= '1' when evn_diff(23 downto 4) = x"00000" or evn_diff(3 downto 0) = "0000" else '0'; 
lrb_match(0) <= htr_time_out when check_next = '0' else lrb_match_q(0);  
-- if NoSkip bit timeout(19) is set, convert skip to start_read
start_read <= matching(3) or (matching(0) and NoSkip); 
check_next <= matching(2); 
missing <= matching(1); 
-- if NoSkip bit timeout(19) is set, convert skip to start_read
skip <= matching(0) and not NoSkip; 
evn_seq <= '1' when (lrb_evn(3 downto 0) - last_evn) = x"1" else '0';
i_lrb_match_q: ram16XNs
    generic map( N => 3)
    port map(
        clk => clk,
        we => we_last_evn, 
        d => lrb_match(3 downto 1),
        a => rp_addr_i,
        o => lrb_match_q);			
i_last_evn: ram16XNs
    generic map( N => 4)
    port map(
        clk => clk,
        we => we_last_evn, 
        d => lrb_evn(3 downto 0),
        a => rp_addr_i,
        o => last_evn);			
we_last_evn <= '1' when strobe = '1' and htr_scan = '1' and (new_rp_dl = '1' or (two_htr_buf = '0' and htr_buf_cnt(0) = '1')) else '0';
-- htr assembling
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			rst_done <= '0';
			en_scan <= '0';
			htr_scan <= '0';
		elsif(strobe = '1')then
			rst_done <= rst_cycle_dl and not rst_cycle;
			en_scan <= ec_header_fifo_wa_h or (en_scan and not htr_scan) or rst_done;
			if(run = '1' and evn_avl = '1' and en_scan = '1' and htr_req_cnt_que(4) = '0')then
				htr_scan <= '1';
			elsif(stop_htr_scan = '1')then
				htr_scan <= '0';
			end if;
    end if;
  end if;
end process;
stop_htr_scan <= (htr_buf_req_i or fake_lrb or missing) and last_lrb;
evn_ra(11 downto 4) <= rp(7 downto 0) when two_htr_buf = '0' or new_rp = '1' or en_scan = '1' else rp_plus_one(7 downto 0);
evn_ra(3 downto 0) <= rp_addr_i;
smry_ra <= rp(7 downto 0) & rp_addr_i;
-- source id and time_out
i_to_cnt: ram16XNs
    generic map( N => 2)
    port map(
        clk => clk,
        we => ec_rp_addr, 
        d => d_to_cnt,
        a => rp_addr_i,
        o => to_cnt);
d_to_cnt(1) <= not rst_cycle and ((to_cnt(0) and (missing xor to_cnt(1))) or (missing and to_cnt(1)));
d_to_cnt(0) <= not rst_cycle and ((not to_cnt(0) and (missing xor to_cnt(1))) or (missing and to_cnt(1)));
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
-- three consecutive timeout then wait no more
      htr_time_out <= ((to_cnt(1) and to_cnt(0)) or htr_wait_ovfl) and not(htr_buf_req_i or fake_lrb or missing);
      ld_evn_wait <= not en_scan or htr_req_cnt_que(4) or not(two_htr_buf or htr_buf_cnt(0));
			if(ld_evn_wait = '1')then
				evn_wait_cntr <= '0' & not timeout(15 downto 8) & "00";
			elsif(EnFakeL1ac = '1')then
				evn_wait_cntr <= evn_wait_cntr + 1;
			end if;
      ld_htr_wait <= htr_buf_req_i or fake_lrb or missing or not htr_scan;
			if(ld_htr_wait = '1')then
				htr_wait_cntr <= '0' & not timeout(7 downto 0) & "0000000";
			elsif(EnHTRtimeout = '1')then
				htr_wait_cntr <= htr_wait_cntr + 1;
			end if;
		end if;
  end if;
end process;
htr_wait_ovfl <= htr_wait_cntr(15);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(we_byte(1) = '1' and addr(14 downto 0) = SYNC_ctrl_addr(16 downto 2))then
        timeout(21 downto 16) <= w_data(21 downto 16);
			end if;
			if(we_byte(2) = '1' and addr(14 downto 0) = SYNC_ctrl_addr(16 downto 2))then
        timeout(15 downto 8) <= w_data(15 downto 8);
			end if;
			if(we_byte(3) = '1' and addr(14 downto 0) = SYNC_ctrl_addr(16 downto 2))then
        timeout(7 downto 0) <= w_data(7 downto 0);
			end if;
-- bits 19-12 evt_stat, bits 11-0 src_id
			if(we_byte(1) = '1' and addr(14 downto 0) = SRC_id_addr(16 downto 2))then
        source_id(19 downto 16) <= w_data(19 downto 16);
			end if;
			if(we_byte(2) = '1' and addr(14 downto 0) = SRC_id_addr(16 downto 2))then
        source_id(15 downto 8) <= w_data(15 downto 8);
			end if;
			if(we_byte(3) = '1' and addr(14 downto 0) = SRC_id_addr(16 downto 2))then
        source_id(7 downto 0) <= w_data(7 downto 0);
			end if;
			if(addr(14 downto 1) = old_cntr_addr(14 downto 1) and old_cntr_addr(0) = '0')then
				rd_counter_high <= '1';
			else
				rd_counter_high <= '0';
			end if;
			addr_q <= addr(1 downto 0);
		end if;
  end if;
end process;
NoL1ac <= timeout(21);
NoWaitNext <= timeout(20);
NoSkip <= timeout(19);
NoMiss <= timeout(18);
EnFakeL1ac <= timeout(17);
EnHTRtimeout <= timeout(16);
timeout(31 downto 22) <= (others => '0');
source_id(31 downto 20) <= (others => '0');
timeout_data <= timeout;
src_id_data <= source_id;
counters <= counter_rb(31 downto 0) when addr_q(0) = '0' else 
            counter_high when rd_counter_high = '1' else
						counter_rb(63 downto 32);
-- DCC error history
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset_err = '1')then
			dcc_err <= (others => '0');
    elsif(strobe = '1')then
			if(TTS = x"1")then
				dcc_err(0) <= '1';
			end if;
			if(TTS = x"4")then
				dcc_err(1) <= '1';
			end if;
			if(TTS = x"2")then
				dcc_err(2) <= '1';
			end if;
--			dcc_err(0) <= dcc_err(0) or TTS(0);
--			dcc_err(1) <= dcc_err(1) or TTS(2);
--			dcc_err(2) <= dcc_err(2) or TTS(1);
			dcc_err(3) <= dcc_err(3) or L1_EvN_mm;
			dcc_err(4) <= dcc_err(4) or L1_BcN_mm;
			dcc_err(5) <= dcc_err(5) or CT_EvN_mm;
			dcc_err(6) <= dcc_err(6) or CT_BcN_mm;
			dcc_err(7) <= dcc_err(7) or inc_bcnterr;
			dcc_err(8) <= dcc_err(8) or inc_serr;
			dcc_err(9) <= dcc_err(9) or inc_derr;
		end if;
  end if;
end process;
-- 32 bit counters
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			evt_bldr_bc <= "00";
    elsif(strobe = '1')then
			evt_bldr_bc(1) <= not ld_evt_bldr_bc and (evt_bldr_bc(1) xor (evt_bldr_bc(0) and wr_tail_dl));
			evt_bldr_bc(0) <= (not ld_evt_bldr_bc and evt_bldr_bc(0)) xor wr_tail_dl;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			slink_bc <= "00";
    elsif(strobe = '1')then
			slink_bc(1) <= not ld_slink_bc and (slink_bc(1) xor (slink_bc(0) and inc_slink_bc));
			slink_bc(0) <= (not ld_slink_bc and slink_bc(0)) xor inc_slink_bc;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			mon_cntr <= "00";
    elsif(strobe = '1')then
			mon_cntr(1) <= not ld_mon_cntr and (mon_cntr(1) xor (mon_cntr(0) and inc_mon_cntr));
			mon_cntr(0) <= (not ld_mon_cntr and mon_cntr(0)) xor inc_mon_cntr;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			rate_OFW_cntr <= "00";
    elsif(strobe = '1')then
			rate_OFW_cntr(1) <= not ld_rate_OFW_cntr and (rate_OFW_cntr(1) xor (rate_OFW_cntr(0) and inc_rate_OFW));
			rate_OFW_cntr(0) <= (not ld_rate_OFW_cntr and rate_OFW_cntr(0)) xor inc_rate_OFW;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			l1ac_cntr <= "00";
    elsif(strobe = '1')then
			l1ac_cntr(1) <= not ld_l1ac_cntr and (l1ac_cntr(1) xor (l1ac_cntr(0) and inc_l1ac));
			l1ac_cntr(0) <= (not ld_l1ac_cntr and l1ac_cntr(0)) xor inc_l1ac;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			CT_EvN_mmc <= "00";
    elsif(strobe = '1')then
			CT_EvN_mmc(1) <= not ld_CT_EvN_mmc and (CT_EvN_mmc(1) xor (CT_EvN_mmc(0) and CT_EvN_mm));
			CT_EvN_mmc(0) <= (not ld_CT_EvN_mmc and CT_EvN_mmc(0)) xor CT_EvN_mm;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			CT_cntr <= "00";
    elsif(strobe = '1')then
			CT_cntr(1) <= not ld_CT_cntr and (CT_cntr(1) xor (CT_cntr(0) and calib_trig));
			CT_cntr(0) <= (not ld_CT_cntr and CT_cntr(0)) xor calib_trig;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			CT_BcN_mmc <= "00";
    elsif(strobe = '1')then
			CT_BcN_mmc(1) <= not ld_CT_BcN_mmc and (CT_BcN_mmc(1) xor (CT_BcN_mmc(0) and CT_BcN_mm));
			CT_BcN_mmc(0) <= (not ld_CT_BcN_mmc and CT_BcN_mmc(0)) xor CT_BcN_mm;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			L1_BcN_mmc <= "00";
    elsif(strobe = '1')then
			L1_BcN_mmc(1) <= not ld_L1_BcN_mmc and (L1_BcN_mmc(1) xor (L1_BcN_mmc(0) and L1_BcN_mm));
			L1_BcN_mmc(0) <= (not ld_L1_BcN_mmc and L1_BcN_mmc(0)) xor L1_BcN_mm;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			L1_EvN_mmc <= "00";
    elsif(strobe = '1')then
			L1_EvN_mmc(1) <= not ld_L1_EvN_mmc and (L1_EvN_mmc(1) xor (L1_EvN_mmc(0) and L1_EvN_mm));
			L1_EvN_mmc(0) <= (not ld_L1_EvN_mmc and L1_EvN_mmc(0)) xor L1_EvN_mm;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			bcnterr_cntr <= "00";
    elsif(strobe = '1')then
			bcnterr_cntr(1) <= not ld_bcnterr_cntr and (bcnterr_cntr(1) xor (bcnterr_cntr(0) and inc_bcnterr));
			bcnterr_cntr(0) <= (not ld_bcnterr_cntr and bcnterr_cntr(0)) xor inc_bcnterr;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			crc_err_cntr <= "00";
    elsif(strobe = '1')then
			crc_err_cntr(1) <= not ld_crc_err_cntr and (crc_err_cntr(1) xor (crc_err_cntr(0) and inc_crc_err));
			crc_err_cntr(0) <= (not ld_crc_err_cntr and crc_err_cntr(0)) xor inc_crc_err;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			scan_cntr_b <= (others => '0');
			rst_counterb <= '1';
    elsif(strobe = '1')then
			scan_cntr_b <= scan_cntr_b + 1;
			if(counterb_addr = x"f")then
				rst_counterb <= '0';
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			ld_evt_bldr_bc <= '0';
			ld_slink_bc <= '0';
			ld_mon_cntr <= '0';
			ld_l1ac_cntr <= '0';
			ld_CT_cntr <= '0';
			ld_CT_EvN_mmc <= '0';
			ld_CT_BcN_mmc <= '0';
			ld_L1_EvN_mmc <= '0';
			ld_L1_BcN_mmc <= '0';
			ld_bcnterr_cntr <= '0';
			ld_rate_OFW_cntr <= '0';
			ld_crc_err_cntr <= '0';
    elsif(strobe = '1')then
			if(scan_cntr_b = "1111")then
				ld_evt_bldr_bc <= '1';
			else
				ld_evt_bldr_bc <= '0';
			end if;
			if(scan_cntr_b = "0000")then
				ld_slink_bc <= '1';
			else
				ld_slink_bc <= '0';
			end if;
			 if(scan_cntr_b = "0001")then
				ld_mon_cntr <= '1';
			else
				ld_mon_cntr <= '0';
			end if;
			 if(scan_cntr_b = "0010")then
				ld_l1ac_cntr <= '1';
			else
				ld_l1ac_cntr <= '0';
			end if;
			 if(scan_cntr_b = "0011")then
				ld_CT_cntr <= '1';
			else
				ld_CT_cntr <= '0';
			end if;
			 if(scan_cntr_b = "0100")then
				ld_CT_EvN_mmc <= '1';
			else
				ld_CT_EvN_mmc <= '0';
			end if;
			 if(scan_cntr_b = "0101")then
				ld_CT_BcN_mmc <= '1';
			else
				ld_CT_BcN_mmc <= '0';
			end if;
			 if(scan_cntr_b = "0110")then
				ld_L1_EvN_mmc <= '1';
			else
				ld_L1_EvN_mmc <= '0';
			end if;
			 if(scan_cntr_b = "0111")then
				ld_L1_BcN_mmc <= '1';
			else
				ld_L1_BcN_mmc <= '0';
			end if;
			 if(scan_cntr_b = "1000")then
				ld_bcnterr_cntr <= '1';
			else
				ld_bcnterr_cntr <= '0';
			end if;
			 if(scan_cntr_b = "1001")then
				ld_rate_OFW_cntr <= '1';
			else
				ld_rate_OFW_cntr <= '0';
			end if;
			 if(scan_cntr_b = "1010")then
				ld_crc_err_cntr <= '1';
			else
				ld_crc_err_cntr <= '0';
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			case scan_cntr_b is
				when "0000" => counterb_inc(1 downto 0) <= evt_bldr_bc;
				when "0001" => counterb_inc(1 downto 0) <= slink_bc;
				when "0010" => counterb_inc(1 downto 0) <= mon_cntr;
				when "0011" => counterb_inc(1 downto 0) <= l1ac_cntr;
				when "0100" => counterb_inc(1 downto 0) <= CT_cntr;
				when "0101" => counterb_inc(1 downto 0) <= CT_EvN_mmc;
				when "0110" => counterb_inc(1 downto 0) <= CT_BcN_mmc;
				when "0111" => counterb_inc(1 downto 0) <= L1_EvN_mmc;
				when "1000" => counterb_inc(1 downto 0) <= L1_BcN_mmc;
				when "1001" => counterb_inc(1 downto 0) <= bcnterr_cntr;
				when "1010" => counterb_inc(1 downto 0) <= rate_OFW_cntr;
				when "1011" => counterb_inc(1 downto 0) <= crc_err_cntr;
				when others => counterb_inc(1 downto 0) <= "00";
			end case;
			counterb_addr <= scan_cntr_b;
			if(addr(14 downto 4) = status_cntr_addr(16 downto 6))then
				status_cntr <= counterb;
			else
				status_cntr <= (others => '0');
			end if;
		end if;
  end if;
end process;
i_counterb: ram16XNd
    generic map( N => 32)
    port map(
        clk => clk,
        we => strobe, 
        d => counterb_in,
        a => counterb_addr,
        dpra => addr(3 downto 0),
        spo => counterb_out,
        dpo => counterb);			
counterb_in <= counterb_inc when rst_counterb = '1' else counterb_out + counterb_inc;
counterb_inc(31 downto 2) <= (others => '0');
-- 64 bit counters
process(clk)
begin
  if(clk'event and clk = '1')then
		if(reset = '1')then
			evt_bldr_wc <= (others => '0');
		elsif(ld_evt_bldr_wc = '1')then
			evt_bldr_wc <= "0000000" & slink_we_i;
		elsif(slink_we_i = '1')then
			evt_bldr_wc <= evt_bldr_wc + 1;
		end if;
    if(ld_slink_wc = '1')then
			slink_wc <= (others => '0');
    elsif(strobe = '1')then
			slink_wc <= slink_wc + inc_slink_wc;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_serr_cntr = '1')then
			serr_cntr <= (others => '0');
    elsif(strobe = '1')then
			serr_cntr <= serr_cntr + inc_serr;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_derr_cntr = '1')then
			derr_cntr <= (others => '0');
    elsif(strobe = '1')then
			derr_cntr <= derr_cntr + inc_derr;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_ready_cntr = '1')then
			ready_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(3 downto 2) = "10")then
			ready_cntr <= ready_cntr + 1;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_busy_cntr = '1')then
			busy_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(3 downto 2) = "01" and run = '1')then
			busy_cntr <= busy_cntr + 1;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_ovfl_cntr = '1')then
			ovfl_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(0) = '1' and TTS(3) = '0')then
			ovfl_cntr <= ovfl_cntr + 1;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_sync_cntr = '1')then
			sync_cntr <= (others => '0');
    elsif(strobe = '1' and TTS(1) = '1' and TTS(3) = '0')then
			sync_cntr <= sync_cntr + 1;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ld_run_cntr = '1')then
			run_cntr <= (others => '0');
    elsif(strobe = '1' and run = '1')then
			run_cntr <= run_cntr + 1;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			scan_cntr <= "0100";
    elsif(strobe = '1' and en_wr_cntr_que_a = '1')then
			if(scan_cntr = "1100")then
				scan_cntr <= "0100";
			else
				scan_cntr <= scan_cntr + 1;
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
      if(scan_cntr(1) = '1')then
				mux4_a <= mux2_a;
				mux4_b <= mux2_c;
			else
				mux4_a <= mux2_b;
				mux4_b <= mux2_d;
			end if;
      if(scan_cntr(2) = '1')then
				cntr_que_a_in(7 downto 0) <= mux4_a;
			else
				cntr_que_a_in(7 downto 0) <= mux4_b;
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(scan_cntr(0) = '0')then
			mux2_a <= "00" & serr_cntr;
			if(scan_cntr(3) = '1')then
				mux2_b <= '0' & run_cntr;
			else
				mux2_b <= evt_bldr_wc;
			end if;
			mux2_c <= '0' & ovfl_cntr;
			mux2_d <= '0' & ready_cntr;
		else
			mux2_a <= "00" & derr_cntr;
			mux2_b <= '0' & slink_wc;
			mux2_c <= '0' & sync_cntr;
			mux2_d <= '0' & busy_cntr;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1') then 
    if(reset = '1')then
			scan_cntr_a <= "0000";
		elsif(strobe = '1')then
			scan_cntr_a <= scan_cntr_a + 1;
		end if;
    if(scan_cntr_a(2 downto 0) = "100" and strobe = '0')then
			scan_cntr_que_a <= '1';
    else
			scan_cntr_que_a <= '0';
    end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			ld_evt_bldr_wc <= '1';
			ld_slink_wc <= '1';
			ld_serr_cntr <= '1';
			ld_derr_cntr <= '1';
			ld_ready_cntr <= '1';
			ld_busy_cntr <= '1';
			ld_ovfl_cntr <= '1';
			ld_sync_cntr <= '1';
			ld_run_cntr <= '1';
    else
			case scan_cntr is
				when "0100" => ld_evt_bldr_wc <= scan_cntr_que_a;
				when "0101" => ld_slink_wc <= scan_cntr_que_a;
				when "0110" => ld_serr_cntr <= scan_cntr_que_a;
				when "0111" => ld_derr_cntr <= scan_cntr_que_a;
				when "1000" => ld_ready_cntr <= scan_cntr_que_a;
				when "1001" => ld_busy_cntr <= scan_cntr_que_a;
				when "1010" => ld_ovfl_cntr <= scan_cntr_que_a;
				when "1011" => ld_sync_cntr <= scan_cntr_que_a;
				when "1100" => ld_run_cntr <= scan_cntr_que_a;
				when others => null;
			end case;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
	   scan_cntr_que_a_dl <= '0';
	   scan_cntr_que_a_dl2 <= '0';
	   en_wr_cntr_que_a <= '0';
	 elsif(strobe = '1')then
	   scan_cntr_que_a_dl <= scan_cntr_que_a;
	   scan_cntr_que_a_dl2 <= scan_cntr_que_a_dl;
	   en_wr_cntr_que_a <= scan_cntr_que_a_dl2;
	 end if;
  end if;
end process;
i_cntr_que_a : srl_fifo
  generic map (N => 12)
  port map(
	rst => reset,
	din => cntr_que_a_in,
	clk => clk,
	w_strobe => strobe,
	we => wr_cntr_que_a,
	r_strobe => '1',
	re => rd_cntr_que_a,
	dav => dav_cntr_que_a,
	dout =>cntr_que_a);
wr_cntr_que_a <= '0' when cntr_que_a_in(7 downto 0) = "0000000" or en_wr_cntr_que_a = '0' else '1';
cntr_que_a_in(11 downto 8) <= scan_cntr(3 downto 0);
i_cntr_que_b : srl_fifo
  generic map (N => 9)
  port map(
	rst => reset,
	din => cntr_que_b_in,
	clk => clk,
	w_strobe => strobe,
	we => wr_cntr_que_b,
	r_strobe => '1',
	re => rd_cntr_que_b,
	dav => dav_cntr_que_b,
	dout =>cntr_que_b);
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			counter_cycle <= "00";
			rd_cntr_que_a <= '0';
			rd_cntr_que_b <= '0';
    else
			counter_cycle(1) <= counter_cycle(0);
			if((dav_cntr_que_b = '1' or dav_cntr_que_a = '1') and counter_cycle = "00" and rst_mm_cntr = '0')then
				counter_cycle(0) <= '1';
			else
				counter_cycle(0) <= '0';
			end if;
      rd_cntr_que_a <= counter_cycle(0) and sel_cntr_que_a;
      rd_cntr_que_b <= counter_cycle(0) and sel_cntr_que_b;
		end if;
		wr_counter <= counter_cycle(1) or not counter_wap(6);
		rst_counter_n <= counter_wap(6);
		if(counter_cycle = "00")then
			sel_cntr_que_b <= dav_cntr_que_b;
			sel_cntr_que_a <= dav_cntr_que_a and not dav_cntr_que_b;
			if(dav_cntr_que_b = '1')then
				increment(10 downto 0) <= "00000000001";
			else
				increment(10 downto 0) <= "000" & cntr_que_a(7 downto 0);
			end if;
		end if;
  end if;
end process;
i_counter_h: RAMB16_S36_S36
	generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => counter_in(63 downto 32),
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => strobe,
  			  WEA => wr_counter,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => counter_wa,
					ADDRB => counter_ra,
					DOA => counter_out(63 downto 32),
					DOB => counter_rb(63 downto 32)
					);
i_counter_l: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => counter_in(31 downto 0),
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => strobe,
  			  WEA => wr_counter,
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => counter_wa,
					ADDRB => counter_ra,
					DOA => counter_out(31 downto 0),
					DOB => counter_rb(31 downto 0)
			  );
-- to avoid write/read conflict
--en_counter_out <= not wr_counter;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(counter_ra(8 downto 6) = "000")then
				counter_high <= counter_rb(63 downto 32);
				old_cntr_addr <= addr(14 downto 0);
			end if;
		end if;
-- after reset counter_wap increments until counter_wap(6) is '1'
-- during this process counters are all reset to zero
-- counter_wap(6) stays '1' and it becomes a register
		if(reset = '1')then
			counter_wap <= (others => '0');
		elsif(counter_cycle = "00")then
			if(counter_wap(6) = '1')then
				counter_wap <= d_counter_wap;
			else
				counter_wap <= counter_wap + 1;
			end if;
		end if;
  end if;
end process;
d_counter_wap <= "100" & cntr_que_b(3 downto 0) when dav_cntr_que_b = '1' else
                 "111" & cntr_que_a(11 downto 8);
counter_wa <= "000" & counter_wap(5 downto 0);
counter_ra(5 downto 0) <= addr(6 downto 1);
counter_ra(8) <= '0' when addr(14 downto 11) = eb_cntr_addr(16 downto 13) else '1';
counter_ra(7) <= '0' when addr(10 downto 9) = eb_cntr_addr(12 downto 11) else '1';
counter_ra(6) <= '0' when addr(8 downto 7) = eb_cntr_addr(10 downto 9) else '1';
counter_in <= (others => '0') when rst_counter_n = '0' else counter_out + increment;
increment(63 downto 11) <= (others => '0');
-- event number mismatch error handling, added 5/11/06
i_mmLUT: ram16XNd
    generic map( N => 8)
    port map(
        clk => clk,
        we => we_mmLUT, 
        d => w_data(7 downto 0),
        a => addr(3 downto 0),
        dpra => mmLUT_scan,
        spo => mmLUT_spo,
        dpo => mmLUT);			
we_mmLUT <= '1' when strobe = '1' and addr(14 downto 3) = mm_ctrl_addr(16 downto 5) and we_byte(3) = '1' else '0';
mmLUT_scan(3) <= '0'	;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(addr(14 downto 3) = mm_ctrl_addr(16 downto 5))then
				mmLUT_data(7 downto 0) <= mmLUT_spo;
			else
				mmLUT_data(7 downto 0) <= x"00";
			end if;
		end if;
		if(reset = '1')then
			bcnt_err <= '0';
		else
			bcnt_err <= inc_bcnterr or (bcnt_err and not inc_oc);
		end if;
		if(reset = '1')then
			bcnt_err_cntr <= (others => '0');
		elsif(inc_oc = '1')then
			if(bcnt_err = '1' and bcnt_err_cntr /= x"f")then
				bcnt_err_cntr <= bcnt_err_cntr + 1;
			elsif(bcnt_err = '0' and bcnt_err_cntr /= x"0")then
				bcnt_err_cntr <= bcnt_err_cntr - 1;
			end if;
		end if;
		if(reset = '1')then
			CT_err_cntr <= (others => '0');
		elsif(ec_header_fifo_wa_h = '1' and evn_out(68) = '1')then
			if(EvN_mm = '1' and CT_err_cntr(3 downto 0) /= x"f")then
				CT_err_cntr(3 downto 0) <= CT_err_cntr(3 downto 0) + 1;
			elsif(EvN_mm = '0' and CT_err_cntr(3 downto 0) /= x"0")then
				CT_err_cntr(3 downto 0) <= CT_err_cntr(3 downto 0) - 1;
			end if;
			if(BCN_mm = '1' and CT_err_cntr(7 downto 4) /= x"f")then
				CT_err_cntr(7 downto 4) <= CT_err_cntr(7 downto 4) + 1;
			elsif(BCN_mm = '0' and CT_err_cntr(7 downto 4) /= x"0")then
				CT_err_cntr(7 downto 4) <= CT_err_cntr(7 downto 4) - 1;
			end if;
		end if;
		if(reset = '1')then
			L1_err_cntr <= (others => '0');
		elsif(ec_header_fifo_wa_h = '1' and evn_out(68) = '0')then
			if(EvN_mm = '1' and L1_err_cntr(3 downto 0) /= x"f")then
				L1_err_cntr(3 downto 0) <= L1_err_cntr(3 downto 0) + 1;
			elsif(EvN_mm = '0' and L1_err_cntr(3 downto 0) /= x"0")then
				L1_err_cntr(3 downto 0) <= L1_err_cntr(3 downto 0) - 1;
			end if;
			if(BCN_mm = '1' and L1_err_cntr(7 downto 4) /= x"f")then
				L1_err_cntr(7 downto 4) <= L1_err_cntr(7 downto 4) + 1;
			elsif(BCN_mm = '0' and L1_err_cntr(7 downto 4) /= x"0")then
				L1_err_cntr(7 downto 4) <= L1_err_cntr(7 downto 4) - 1;
			end if;
		end if;
		if(ec_header_fifo_wa_h = '1')then
			mmLUT_scan(2 downto 0) <= "000";
		elsif(mmLUT_scan(2) = '0')then
			mmLUT_scan(2 downto 0) <= mmLUT_scan(2 downto 0) + 1;
		end if;
		if(mmLUT_scan(2) = '1')then
			err_cntr <= bcnt_err_cntr;
		else
			case mmLUT_scan(1 downto 0) is
				when "00" => err_cntr <= CT_err_cntr(3 downto 0);
				when "01" => err_cntr <= CT_err_cntr(7 downto 4);
				when "10" => err_cntr <= L1_err_cntr(3 downto 0);
				when "11" => err_cntr <= L1_err_cntr(7 downto 4);
				when others => null;
			end case;
		end if;
		threshold <= mmLUT(7 downto 4);	 
		stateToGo(1) <= mmLUT(3) and mmLUT(2) and (mmLUT(1) xnor mmLUT(0));
		stateToGo(0) <= (mmLUT(3) and mmLUT(2) and mmLUT(1) and mmLUT(0)) or (not mmLUT(3) and not mmLUT(2) and mmLUT(1) and not mmLUT(0));
		if(err_cntr > threshold)then
			ttc_state <= stateToGo;	 
		else
			ttc_state <= "00";
		end if;
		if(ttc_state(1) = '1' or HTR_state(1) = '1')then
			err_state(1) <= '1';	 
			elsif(strobe = '1')then
			err_state(1) <= '0';
		end if;
		if((ttc_state(0) = '1' and (ttc_state(1) = '1' or HTR_state(1) = '0')) or (HTR_state(0) = '1' and (HTR_state(1) = '1' or ttc_state(1) = '0')))then
			err_state(0) <= '1';	 
		elsif(strobe = '1')then
			err_state(0) <= '0';
		end if;
  end if;
end process;
mmLUT_data(31 downto 8) <= x"000000";
faked_evn <= evn_out(89);
process(addr,evn_out)
begin
	if(addr(14 downto 12) = evn_buf_addr(16 downto 14))then
		case addr_q is
			when "00" => evn_data <= x"00" & evn_out(23 downto 0);
			when "01" => evn_data <= faked_evn & "000" & evn_out(83 downto 68) & evn_out(35 downto 24);
			when "10" => evn_data <= evn_out(67 downto 36);
			when others => evn_data <= x"00000000";
		end case;
	else
		evn_data <= (others => '0');
	end if;
end process;
evn_pntr <= "00" & evn_fifo_wa & "000000" & evn_fifo_ra & x"0";
-- event number mismatch counters, added 5/13/08
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(htr_scan = '1' and ((start_read = '1' and bc_evn_match(2 downto 0) /= "111") or missing = '1' or skip = '1'))then
				wr_cntr_que_b <= '1';
			else
				wr_cntr_que_b <= '0';
			end if;
			if(missing = '1' or skip = '1')then
				cntr_que_b_in <= "1000" & missing & rp_addr_i;
			else
--				cntr_que_b_in <= '0' & not bc_evn_match(2 downto 0) & bc_evn_match(3) & rp_addr_i;
				cntr_que_b_in <= '0' & not bc_evn_match(2 downto 0) & '0' & rp_addr_i;
			end if;
		end if;
  end if;
end process;
i_evn_mm_cntr: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => evn_mm_cntr_in,
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => '1',
  			  WEA => wr_evn_mm_cntr(5),
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => evn_mm_cntr_aa,
					ADDRB => evn_mm_cntr_ab,
					DOB => evn_mm_cntr_out
					);
evn_mm_cntr_ab(8 downto 7) <= "00";
g_evn_mm_cntr_ab_dl: for i in 0 to 6 generate
	i_evn_mm_cntr_ab_dl:
		delayN generic map(N => 2) port map(c => clk, d => evn_mm_cntr_ab(i), ce => '1', q => evn_mm_cntr_ab_dl(i));
end generate;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(reset = '1')then
			rst_mm_cntr <= '1';
		elsif(evn_mm_cntr_aa(6 downto 0) = "1111111")then
			rst_mm_cntr <= '0';
		end if;
		if(rst_mm_cntr = '1')then
			evn_mm_cntr_in <= (others => '0');
		else
			evn_mm_cntr_in <= evn_mm_cntr_out_q + 1;
		end if;
		if(reset = '1')then
			evn_mm_cntr_aa <= (others => '0');
		elsif(rst_mm_cntr = '1')then
			evn_mm_cntr_aa(6 downto 0) <= evn_mm_cntr_aa(6 downto 0) + 1;
		else
			evn_mm_cntr_aa(6 downto 0) <= evn_mm_cntr_ab_dl;
		end if;	 
		evn_mm_cntr_out_q <= evn_mm_cntr_out;
		if(rd_cntr_que_b = '1')then
			evn_mm_cntr_ab(6 downto 0) <= cntr_que_b(4) & cntr_que_b(8) & cntr_que_b(8) & cntr_que_b(3 downto 0);
		elsif(evn_mm_cntr_ab(5) = '0')then
			evn_mm_cntr_ab(5) <= evn_mm_cntr_ab(5) xor evn_mm_cntr_ab(4);
			evn_mm_cntr_ab(4) <= not evn_mm_cntr_ab(4);
		end if;
		if(rst_mm_cntr = '1')then
			wr_evn_mm_cntr(5) <= '1';
		else
			wr_evn_mm_cntr(5) <= wr_evn_mm_cntr(4);
		end if;
		if(rd_cntr_que_b = '1')then
			wr_evn_mm_cntr(2 downto 0) <= (cntr_que_b(8) or cntr_que_b(5)) & cntr_que_b(6) & cntr_que_b(7);
		else
			wr_evn_mm_cntr(4 downto 0) <= wr_evn_mm_cntr(3 downto 0) & '0';
		end if;
  end if;
end process;
i_evn_mm_buf: RAMB16_S36_S36
	  generic map(SIM_COLLISION_CHECK => "NONE")
  port map(DIA => evn_mm_cntr_in,
  			  DIPA => x"0",
  			  DIB => x"00000000",
  			  DIPB => x"0",
  			  ENA => '1',
  			  ENB => strobe,
  			  WEA => wr_evn_mm_cntr(5),
  			  WEB => '0',
  			  SSRA => '0',
  			  SSRB => '0',
					CLKA => clk,
					CLKB => clk,
					ADDRA => evn_mm_cntr_aa,
					ADDRB => evn_mm_cntr_a,
					DOB => evn_mm_cntr
					);
evn_mm_cntr_a(6 downto 0) <= addr(6 downto 0);
evn_mm_cntr_a(8) <= '0' when addr(14 downto 11) = evn_mmc_addr(16 downto 13) else '1';
evn_mm_cntr_a(7) <= '0' when addr(10 downto 7) = evn_mmc_addr(12 downto 9) else '1';
-- debugging
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(addr(14 downto 6) = eb_dbg_addr(16 downto 8))then
				case addr(1 downto 0) is
					when "00" =>
						debug(15 downto 0) <= rp_addr_i & htr_scan & clear & '0' & rp;
						debug(31 downto 16) <= wp_addr & evn_avl & en_scan & '0' & dpo_wp;
					when "01" =>
						debug(11 downto 0) <= matching & '0' & header_fifo_ra(6 downto 0);
						debug(23 downto 12) <= lrb_match;
						debug(31 downto 24) <= '0' & header_sent & trailer_sent & htr_rd_cnt_i;
					when "10" => debug <= summary;
					when others => debug <= x"00" & "000" & htr_req_cnt_que & x"000" & htr_req_cnt;
				end case;
			else
				debug <= (others => '0');
			end if;
		end if;
  end if;
end process;
end my_arch;
