library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;
entity led_driver is
    Port ( clk : in std_logic;
           locked : in std_logic;
           n_strobe : in std_logic;
           test_led : in std_logic;
           err_in : in std_logic_vector(24 downto 0);
           led_in : in std_logic_vector(24 downto 0);
					 update_led : out std_logic;
           led : out std_logic_vector(24 downto 0)
           );
end led_driver;

architecture my_arch of led_driver is
signal locked_dl : std_logic := '0';
signal strobe : std_logic := '0';
signal fiveMHz : std_logic := '0';
signal div : std_logic_vector(19 downto 0) := (others => '0');
signal startup_delay : std_logic_vector(1 downto 0) := (others => '0');
signal update : std_logic := '0';
signal blink : std_logic := '0';
signal half_sec : std_logic := '0';
signal five_sec : std_logic := '0';
signal ce_five_sec : std_logic := '0';
signal err_rst : std_logic := '0';
signal err_seen: std_logic_vector(24 downto 0) := (others => '0');
signal error: std_logic_vector(24 downto 0) := (others => '0');
signal latch: std_logic_vector(24 downto 0) := (others => '0');
signal tp: std_logic_vector(24 downto 0) := (others => '0');
begin
led <= latch;
update_led <= update;
--	using lsr as clock divider to generate 5MHz signal
i_fiveMHz: delayN generic map(N => 15,INIT_VALUE => x"0001") port map(d => fiveMHz, ce => strobe, c => clk, q => fiveMHz);
i_half_sec: delayN generic map(N => 10,INIT_VALUE => x"0003") port map(d => half_sec, ce => div(19), c => clk, q => half_sec);
i_five_sec: delayN generic map(N => 10,INIT_VALUE => x"0001") port map(d => five_sec, ce => ce_five_sec, c => clk, q => five_sec);
ce_five_sec <= strobe and div(19) and half_sec;
err_rst <= div(19) and half_sec and five_sec;
i_rst_VMEclk : SRL16 generic map(INIT => x"0000")
   port map (
      Q => locked_dl,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '1',     -- Select[2] input
      A3 => '1',     -- Select[3] input
      CLK => clk,   -- Clock input
      D => locked        -- SRL data input
   );
process(clk,locked)
begin
	if(locked = '0')then
		strobe <= '0';
  elsif(clk'event and clk = '1')then
		strobe <= n_strobe;
	end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			update <= div(19) and startup_delay(1);
			if(div(19) = '1')then
				div <= x"05dd0";
			elsif(fiveMHz = '1')then
				div <= div + 1;
			end if;
			if(half_sec = '1' and div(19) = '1')then
				startup_delay(1) <= startup_delay(1) or startup_delay(0);
				startup_delay(0) <= not startup_delay(0);
			end if;
			if(test_led = '0')then
				tp <= x"000000" & '1';
      elsif(div(19) = '1' and half_sec = '1' and blink = '1')then
				tp <= tp(23 downto 0) & tp(24);
			end if;	
-- blibk in one second period to indicate error conditions
      if(div(19) = '1')then
				blink <= blink xor half_sec;
			end if;	
-- for none persisting errors, error blinking mode lasts five to ten senconds
			if(err_rst = '1')then
				err_seen <= error xor err_seen; 
			end if;
			for i in error'range loop
				error(i) <= err_in(i) or (error(i) and not (err_rst and err_seen(i)));
			end loop;
			for i in latch'range loop
				if(test_led = '1')then
					latch(i) <= tp(i);
				elsif(error(i) = '1')then
					latch(i) <= blink;
				else
					latch(i) <= led_in(i) or (latch(i) and not update);
				end if;
			end loop;
		end if;
	end if;
end process;
end my_arch;
