library IEEE,mylib;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;

entity slink is
    port (
-- main clock 128MHz
        clk: in STD_LOGIC;
-- resets slink, active high
        reset: in STD_LOGIC;
-- slink monitoring
        n_strobe: in STD_LOGIC;
        addr: in STD_LOGIC_VECTOR (15 downto 0);
        we_byte : in std_logic_vector(3 downto 0);
        w_data: in STD_LOGIC_VECTOR (31 downto 0);
        slink_buf: out STD_LOGIC_VECTOR (31 downto 0);
        slink_pntr: out STD_LOGIC_VECTOR (31 downto 0);
-- data in from dcc logic
        en_slink: in STD_LOGIC;
        slink64: in STD_LOGIC;
-- header and trailer are required to be marked as control data
        slink_ctrl: in STD_LOGIC;
        slink_din: in STD_LOGIC_VECTOR (35 downto 0);
-- data in write enable
        slink_we: in STD_LOGIC;
--	signal for LED display
        slink_send: out STD_LOGIC;
-- fifo full
        slink_full: out STD_LOGIC;
-- link down output
        slink_down: out STD_LOGIC;
-- slink word count output
        inc_slink_wc: out STD_LOGIC;
        inc_slink_bc: out STD_LOGIC;
        inc_crc_err: out STD_LOGIC;
-- slink test in
        slink_test: in STD_LOGIC;
-- SLINK interface
				LDOWNn : in  STD_LOGIC;
				LFFn : in  STD_LOGIC;
				LRL : in  STD_LOGIC_VECTOR (3 downto 0);
				LSF : in  STD_LOGIC_VECTOR (3 downto 0);
				UCLK : out  STD_LOGIC;
				UTESTn : out  STD_LOGIC;
				UWENn : out  STD_LOGIC;
				URESETn : out  STD_LOGIC;
				UCTRLn : out  STD_LOGIC;
				UDW : out  STD_LOGIC_VECTOR (1 downto 0);
				UD : out  STD_LOGIC_VECTOR (63 downto 0);
				USF : out  STD_LOGIC_VECTOR (3 downto 0));
end slink;

architecture slink_arch of slink is
signal strobe: std_logic := '0';
signal was_data: std_logic := '0';
signal slink_re: std_logic := '0';
signal uwe: std_logic := '0';
signal urstn: std_logic := '0';
signal urstn_dl: std_logic := '0';
signal buf_empty: std_logic := '0';
signal buf_we: std_logic_vector(3 downto 0) := (others => '0');
signal slink_up: std_logic := '0';
signal slink_down_i: std_logic := '0';
signal buf_wa: std_logic_vector(13 downto 0) := (others => '0');
signal buf_ra: std_logic_vector(13 downto 0) := (others => '0');
signal word_cnt: std_logic_vector(7 downto 0) := (others => '0');
signal buf_din: std_logic_vector(31 downto 0) := (others => '0');
signal buf_dout: std_logic_vector(31 downto 0) := (others => '0');
signal buf_cin: std_logic_vector(3 downto 0) := (others => '0');
signal buf_cout: std_logic_vector(3 downto 0) := (others => '0');
signal rst_cycle: std_logic_vector(9 downto 0) := (others => '0');
signal udp: std_logic_vector(63 downto 0) := (others => '0');
signal crc_data: std_logic_vector(63 downto 0) := (others => '0');
signal event_crc: std_logic_vector(15 downto 0) := (others => '0');
signal crc: std_logic_vector(15 downto 0) := (others => '0');
signal trailer: std_logic := '0';
signal padding: std_logic := '0';
signal last_word: std_logic := '0';
signal init_crcp: std_logic := '0';
signal init_crc: std_logic := '0';
signal chk_crc: std_logic := '0';
signal we_crcp: std_logic := '0';
signal we_crc: std_logic := '0';
signal cntr: std_logic_vector(4 downto 0) := (others => '0');
begin
slink_send <= uwe;
slink_down <= slink_down_i;
-- after changing udw, must reset slink!!!
udw <= "11" when slink64 = '1' else "00";
URESETn <= urstn;
usf <= "0000";
USF <= x"0";
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
		uclk <= not strobe;
		if(slink_we = '1' and slink_din(34 downto 32) = "101")then
			trailer <= '1';
		else
			trailer <= '0';
		end if;
		padding <= trailer and not buf_wa(5);
		if(reset = '1')then
			was_data <= '0';
		elsif(uwe = '1' and strobe = '1')then
			was_data <= not buf_cout(0) and not last_word;
		end if;
    if(strobe = '1')then
			inc_slink_wc <= uwe;
-- control word after data is end of event
			inc_slink_bc <= uwe and was_data and buf_cout(0);
      uwe <= slink_re;
      UWENn <= not uwe or last_word;
      utestn <= not slink_test;
      uctrln <= not buf_cout(0);
			ud <= udp;
		end if;
  end if;
end process;
-- The low word is always registered to delay one clock cycle
udp(63 downto 32) <= buf_dout;
process(clk)
begin
  if(clk'event and clk = '1')then
	  udp(31 downto 0) <= buf_dout;
		if(slink_up = '1')then
			rst_cycle <= (others => '0');
		elsif(strobe = '1')then
			rst_cycle <= rst_cycle + 1;
		end if;
  end if;
end process;
slink_up <= not slink_down_i or not en_slink or slink_test;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			slink_down_i <= '1';
			urstn <= '0';
    elsif(strobe = '1')then
			slink_down_i <= not ldownn;
-- It is not mentioned in the slink spec what state urstn should be in test mode
--	   urstn <= not slink_down_i and (urstn or not urstn_dl) and en_slink;
			urstn <= not rst_cycle(9) and (urstn or not urstn_dl);
		end if;
  end if;
end process;
i_urstn_dl: delayN
  generic map(N => 9) port map( d => urstn, ce => strobe, c => clk, q => urstn_dl );
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			buf_empty <= '1';
			slink_full <= '0';
		elsif(strobe = '1')then
			if(word_cnt(7 downto 1) = "0000000")then
				buf_empty <= not word_cnt(0) or slink_re;
			else
				buf_empty <= '0';
			end if;
-- leave at least 128 words for 64 words htr_burst
			slink_full <= word_cnt(7) and word_cnt(6) and en_slink;
		end if;
    if(reset = '1')then
			slink_re <= '0';
		elsif(strobe = '1')then
			slink_re <= slink_re and slink64;
		else
			slink_re <=  lffn and urstn and urstn_dl and not buf_empty and en_slink and ldownn and not(uwe and was_data and buf_cout(0));
--			slink_re <=  not buf_empty and en_slink and cntr(4) and cntr(2) and not(uwe and was_data and buf_cout(0));
		end if;
		if(strobe = '1')then
			cntr <= cntr + 1;
		end if;
  end if;
end process;
-- when not in run mode, data can be written to	SLINK from pci3 interface
process(clk)
begin
  if(clk'event and clk = '1')then
    if(slink_we = '1')then
			buf_din <= slink_din(31 downto 0);
		else
			buf_din <= w_data;
		end if;
		buf_cin(0) <= slink_ctrl;
		if(en_slink = '0')then
			buf_we <= x"0";
		elsif(slink_we = '1' and slink_din(35) = '0')then
			buf_we <= x"f";
		elsif(strobe = '1' and addr(14 downto 9) = SL_buf_addr(16 downto 11))then
			buf_we <= we_byte;
		else
			buf_we <= x"0";
		end if;
		if(reset = '1')then
			buf_wa <= (others => '0');
		elsif(buf_we(3) = '1' or padding = '1')then
			buf_wa(13 downto 5) <= buf_wa(13 downto 5) + 1;
		end if;
		if(reset = '1')then
			buf_ra <= (others => '0');
		elsif(en_slink = '0')then
			buf_ra(13 downto 5) <= addr(8 downto 0);
		elsif(slink_re = '1')then
			buf_ra(13 downto 5) <= buf_ra(13 downto 5) + 1;
		end if;
		if(strobe = '1')then
			if(addr(14 downto 0) = SL_pntr_addr(16 downto 2))then
				slink_pntr <= "0000000" & buf_wa(13 downto 5) & "0000000" & buf_ra(13 downto 5);
			else
				slink_pntr <= (others => '0');
			end if;
		end if;
  end if;
end process;
buf_cin(3 downto 1) <= "000";
-- slink data buffer. When SLINK is not enabled, the last 512 words 
-- can be read back through pci3 interface for debugging
i_slink_buf: RAMB16BWER
   generic map (
      DATA_WIDTH_A => 36,  -- Valid values are 0, 1, 2, 4, 9, 18, or 36
      DATA_WIDTH_B => 36,  -- Valid values are 0, 1, 2, 4, 9, 18, or 36
			SIM_COLLISION_CHECK => "NONE")
     port map(
        dia => x"00000000", 
        dipa => x"0", 
        addra => buf_ra,
        clka => clk, 
        rsta => '0', 
        ena => '1', 
        wea => x"0",
				doa => buf_dout,
				dopa => buf_cout,
        dib => buf_din,
        dipb => buf_cin,
        addrb => buf_wa,
        clkb => clk,
        rstb => '0',
				REGCEA => '0', -- 1-bit A port output register enable input
				REGCEB => '0', -- 1-bit B port output register enable input
        enb => '1',
        web => buf_we);
-- Only send pairs of 32-bit words, even if data width are 32-bits
word_cnt <= buf_wa(13 downto 6) - buf_ra(13 downto 6);
slink_buf <= buf_dout when addr(14 downto 9) = SL_buf_addr(16 downto 11) else (others => '0');
-- CRC checek
i_crc: crc16D64
	port map(
		clk => clk,
		init_crc => init_crc,
		we_crc => we_crc,
		d => crc_data,
		crc => crc);
init_crc <= init_crcp and strobe;
we_crc <= we_crcp and strobe;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			last_word <= uwe and was_data and buf_cout(0);
			chk_crc <= last_word;
			if(chk_crc = '1' and crc /= event_crc)then
				inc_crc_err <= '1';
			else
				inc_crc_err <= '0';
			end if;
			if(reset = '1' or last_word = '1')then
				init_crcp <= '1';
			elsif(uwe = '1')then
				init_crcp <= '0';
			end if;
			if(was_data = '1' and buf_cout(0) = '1')then
				crc_data <= udp(63 downto 32) & x"0000" & udp(15 downto 0);
				event_crc <= udp(31 downto 16);
			else
				crc_data <= udp;
			end if;
			we_crcp <= uwe;
		end if;
  end if;
end process;
end slink_arch;
