-- TTCrx interface module
-- TTCrx register file occupies 128 bytes of memory space
-- to simplify the design, all accesses are 32 bit wide,
-- although only the LSB byte carries data.
-- according to TTCrx manual, its SCL line is input only.
-- So no arbitration and no wait holding are assumed in this design. 
library IEEE,mylib;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use mylib.dcc_pack.all;
library UNISIM;
use UNISIM.VComponents.all;
entity ttc_if is
--		generic (rate_limit : std_logic_vector(9 downto 0) := "0111110010");
    Port (
	 		  clk : in std_logic;
	 		  reset : in std_logic;
	 		  run : in std_logic;
	 		  n_strobe : in std_logic;
				rst_ttc : in std_logic;
-- bcn offset
				TTC_ID : out std_logic_vector(31 downto 0);
				TTC_BCN_off : out std_logic_vector(31 downto 0);
				TTC_cal_win : out std_logic_vector(31 downto 0);
				TTC_i2c_data : out std_logic_vector(31 downto 0);
        we_byte : in std_logic_vector(3 downto 0);
				w_data : in std_logic_vector(31 downto 0);
				addr : in std_logic_vector(15 downto 0);
-- TTCrx status : ready, single error and double error
        en_brcst : in std_logic;
	 		  ttc_start : out std_logic;
	 		  ttc_stop : out std_logic;
	 		  ttc_soft_reset : out std_logic;
	 		  ttc_ready : out std_logic;
	 		  ttc_serr : out std_logic;
	 		  ttc_derr : out std_logic;
	 		  ttc_bcnt_err : out std_logic;
	 		  rate_OFW : out std_logic;
	 		  sync_lost : out std_logic;
        inc_oc : out std_logic;
        inc_l1ac : out std_logic;
        inc_bcnterr : out std_logic;
        inc_serr : out std_logic;
        inc_derr : out std_logic;
-- evn_fifo_full is controlled by strobe signal!!!
        state : in std_logic_vector(3 downto 0);
	 		  evn_fifo_full : in std_logic;
	 		  ttc_evcnt_reset : out std_logic;
	 		  event_number_avl : out std_logic;
        event_number : out std_logic_vector(59 downto 0);
-- ttcrx signals
				ttc_clk : in  STD_LOGIC;
				T_ttcready : in  STD_LOGIC;
				T_l1accept : in  STD_LOGIC;
				T_brcststr : in  STD_LOGIC;
				T_brcst : in  STD_LOGIC_VECTOR (7 downto 2);
				T_sinerrstr : in  STD_LOGIC;
				T_dberrstr : in  STD_LOGIC;
				T_evcntres : in  STD_LOGIC;
				T_bcntres : in  STD_LOGIC;
				T_bcntstr : in  STD_LOGIC;
				T_reset_b : out  STD_LOGIC;
				T_scl : out  STD_LOGIC;
				T_sda : inout  STD_LOGIC;
				T_id : out  STD_LOGIC_VECTOR (13 downto 0)
        );
end ttc_if;

architecture ttc_arch of ttc_if is
signal ttc_brcst : std_logic_vector(3 downto 0) := (others => '0');
signal ttc_sinerrstr : std_logic := '0';
signal ttc_dberrstr : std_logic := '0';
signal ttc_l1accept : std_logic := '0';
signal ttc_bcntres : std_logic := '0';
signal ttc_evcntres : std_logic := '0';
signal ttc_OCres : std_logic := '0';
signal ttc_l1accept_dl : std_logic := '0';
signal ttc_bcntres_dl : std_logic := '0';
signal ttc_OCres_dl : std_logic := '0';
signal dl_a : std_logic_vector(3 downto 0) := (others => '0');
signal dl_b : std_logic_vector(3 downto 0) := (others => '0');
signal bcnt : std_logic_vector(11 downto 0) := (others => '0');
--signal evcnt : std_logic_vector(23 downto 0) := (others => '0');
signal oc : std_logic_vector(31 downto 0) := (others => '0');
signal strobe : std_logic := '0';
signal en_bcnt_res : std_logic := '0';
signal en_bcnt_err : std_logic := '0';
signal brcst_GapTrig : std_logic := '0';
signal brcst_GapPed : std_logic := '0';
signal brcst_GapLaser : std_logic := '0';
signal rst_bcnt : std_logic := '0';
signal cal_win : std_logic := '0';
signal cal_winReg : std_logic_vector(31 downto 0) := x"8da60d9d";
signal calType : std_logic_vector(3 downto 0) := (others => '0');
signal Laser_TO : std_logic_vector(13 downto 0) := (others => '0');
signal clk_div : std_logic_vector(7 downto 0) := (others => '0');
signal ec_i2c : std_logic := '0';
signal i2c_busy : std_logic := '0';
signal ttc_id_i : std_logic_vector(31 downto 0) := (others => '0');
signal bcn_offs1 : std_logic_vector(11 downto 0) := (others => '0');
signal bcn_off : std_logic_vector(31 downto 0) := (others => '0');
signal i2c_ad : std_logic_vector(15 downto 0) := (others => '0');
signal i2c_data : std_logic_vector(31 downto 0) := (others => '0');
signal i2c_active : std_logic := '0';
signal i2c_fail : std_logic := '0';
signal data_cycle : std_logic := '0';
signal i2c_phase : std_logic_vector(2 downto 0) := (others => '0');
signal i2c_done : std_logic := '0';
signal i2c_cntr : std_logic_vector(4 downto 0) := (others => '0');
signal rst_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal scl : std_logic := '0';
signal sda : std_logic_vector(17 downto 0) := (others => '0');
signal eoc : std_logic := '0';
signal event_number_avl_i : std_logic := '0';
signal dec_rate_cntr : std_logic := '0';
signal rate_div : std_logic_vector(7 downto 0) := (others => '0');
signal rate_cntr : std_logic_vector(5 downto 0) := (others => '0');
signal ttc_sync_wa : std_logic_vector(3 downto 0) := (others => '0');
signal ttc_sync_ra : std_logic_vector(3 downto 0) := (others => '0');
signal ttc_sync_din : std_logic_vector(10 downto 0) := (others => '0');
signal ttc_sync_dout : std_logic_vector(10 downto 0) := (others => '0');
signal ttc_sync_wa1_sync : std_logic_vector(1 downto 0) := (others => '0');
signal ttc_sync_wa0_sync : std_logic_vector(1 downto 0) := (others => '0');
signal ttc_sync_do : std_logic_vector(10 downto 0) := (others => '0');
signal ttc_sync_do_val : std_logic := '0';
begin
event_number_avl <= event_number_avl_i;
inc_l1ac <= ttc_l1accept_dl;
inc_serr <= ttc_sinerrstr;
inc_derr <= ttc_dberrstr;
ttc_ready <= T_ttcready;
T_id <= ttc_id_i(13 downto 0);
ttc_id_i(31 downto 25) <= "0000000";
ttc_id_i(15 downto 14) <= "00";
bcn_off(31 downto 13) <= x"0000" & "000";
cal_winreg(30 downto 22) <= "000110110";
cal_winreg(15 downto 6) <= calType & "110110";
-- ttc input registers
ttc_OCres <= '1' when ttc_sync_din(4 downto 0) = "10011" else '0';
-- adjusting Bcnt delay
dl_a <= ttc_id_i(23 downto 20) when ttc_id_i(24) = '1' else x"0";
dl_b <= ttc_id_i(23 downto 20) when ttc_id_i(24) = '0' else x"0";
i_ttc_OCres_dl: srlc16 port map(
            d => ttc_OCres,
						clk => ttc_clk,
						a0 => dl_a(0),
						a1 => dl_a(1),
						a2 => dl_a(2),
						a3 => dl_a(3),
						q => ttc_sync_din(10));
i_ttc_bcntres_dl: srlc16 port map(
            d => ttc_bcntres,
						clk => ttc_clk,
						a0 => dl_a(0),
						a1 => dl_a(1),
						a2 => dl_a(2),
						a3 => dl_a(3),
						q => ttc_sync_din(6));
i_ttc_l1accept_dl: srlc16 port map(
            d => ttc_l1accept,
						clk => ttc_clk,
						a0 => dl_b(0),
						a1 => dl_b(1),
						a2 => dl_b(2),
						a3 => dl_b(3),
						q => ttc_sync_din(7));
i_ttc_evcntres_dl: srlc16 port map(
            d => ttc_evcntres,
						clk => ttc_clk,
						a0 => dl_b(0),
						a1 => dl_b(1),
						a2 => dl_b(2),
						a3 => dl_b(3),
						q => ttc_sync_din(5));
process(ttc_clk)
begin
  if(ttc_clk'event and ttc_clk = '1')then
    ttc_sync_din(9) <= T_sinerrstr;
    ttc_sync_din(8) <= T_dberrstr;
		ttc_l1accept <= T_l1accept;
		ttc_bcntres <= T_bcntres;
		ttc_evcntres <= T_evcntres;
		ttc_sync_din(4) <=	T_brcststr;
		ttc_sync_din(3 downto 0) <= T_brcst(7 downto 5) & T_brcst(3);
		ttc_sync_wa(1 downto 0) <= ttc_sync_wa(0) & not ttc_sync_wa(1);
  end if;
end process;
i_ttc_sync: ram16XNd generic map( N => 11)
    port map(
        clk => ttc_clk,
        we	=> '1',
        d		=> ttc_sync_din,
        a		=> ttc_sync_wa,
        dpra=> ttc_sync_ra,
        dpo	=> ttc_sync_dout
    );
ttc_sync_wa(3 downto 2) <= "00";
ttc_sync_ra(3 downto 2) <= "00";
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			ttc_sync_wa1_sync <= ttc_sync_wa1_sync(0) & ttc_sync_wa(1);
			ttc_sync_wa0_sync <= ttc_sync_wa0_sync(0) & ttc_sync_wa(0);
			if(ttc_sync_wa1_sync(1) /= ttc_sync_ra(1) or ttc_sync_wa0_sync(1) /= ttc_sync_ra(0))then
				ttc_sync_ra(1 downto 0) <= ttc_sync_ra(0) & not ttc_sync_ra(1);
				ttc_sync_do_val <= '1';
			else
				ttc_sync_do_val <= '0';
			end if;
			ttc_sync_do <= ttc_sync_dout;
		end if;
  end if;
end process;
-- reset ttcrx after ttc_id_i is valid
process(clk)
begin
  if(clk'event and clk = '1')then
    strobe <= n_strobe;
    if(strobe = '1' and rst_ttc = '1')then
			T_reset_b <= '0';
		elsif(ec_i2c = '1' and rst_cnt(1) = '1')then
			T_reset_b <= '1';
		end if;
		ec_i2c <= clk_div(7) and n_strobe;
    if(strobe = '1' and rst_ttc = '1')then
			rst_cnt <= "00";
		elsif(ec_i2c = '1')then
			rst_cnt <= rst_cnt + 1;
		end if;
-- this divider scales down the clock to 1MHz
		if(strobe = '1')then
			if(clk_div(7) = '1')then
				clk_div <= x"35";
			else
				clk_div <= clk_div + 1;
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(ttc_sync_do_val = '0')then
				ttc_OCres_dl <= '0';
				ttc_sinerrstr <= '0';
				ttc_dberrstr <= '0';
				ttc_l1accept_dl <= '0';
--				ttc_bcntres_dl <= '0';
				ttc_evcnt_reset <= '0';
				ttc_brcst <= x"0";
			else
				ttc_OCres_dl <= ttc_sync_do(10);
				ttc_sinerrstr <= ttc_sync_do(9);
				ttc_dberrstr <= ttc_sync_do(8);
				ttc_l1accept_dl <= ttc_sync_do(7);
--				ttc_bcntres_dl <= ttc_sync_do(6);
				ttc_evcnt_reset <= ttc_sync_do(5);
				if(ttc_sync_do(4) = '1')then
					ttc_brcst <= ttc_sync_do(3 downto 0);
					i2c_data(31 downto 29) <= ttc_sync_do(3 downto 1);
					i2c_data(27) <= ttc_sync_do(0);
				else
					ttc_brcst <= x"0";
				end if;
			end if;
			if(ttc_sync_do_val = '1')then
				ttc_bcntres_dl <= ttc_sync_do(6);
				if(bcnt = x"dea")then
					rst_bcnt <= '1';
				else
					rst_bcnt <= '0';
				end if;
			end if;
			if(ttc_sync_do_val = '1')then
				if(ttc_bcntres_dl = '1' and en_bcnt_res = '1')then
					bcnt <= bcn_off(11 downto 0);
				elsif(rst_bcnt = '1')then
					bcnt <= x"000";
				else
					bcnt <= bcnt + 1;
				end if;
			end if;
			if(ttc_OCres_dl = '1')then
				oc <= x"0000000" & ttc_id_i(19 downto 16);
			elsif(rst_bcnt = '1' and ttc_sync_do_val = '1')then
				oc <= oc + 1;
			end if;
			inc_oc <= ttc_bcntres_dl;
			if(bcnt = cal_winreg(11 downto 0))then
				cal_win <= '1';
			elsif(bcnt(5 downto 0) = cal_winreg(21 downto 16))then
				cal_win <= '0';
			end if;
			if(bcn_off(11 downto 0) = x"000")then
				bcn_offs1 <= x"deb";
			else
				bcn_offs1 <= bcn_off(11 downto 0) - 1;
			end if;
			if(reset = '1' or T_ttcready = '0')then -- modified for dsp_chip301f
				en_bcnt_err <= '0';
			elsif(ttc_bcntres_dl = '1' or bcnt = bcn_offs1)then
				en_bcnt_err <= '1';
			end if;
			if(reset = '1')then
				inc_bcnterr <= '0';
				ttc_bcnt_err <= '0';
			elsif(((ttc_bcntres_dl = '1' and bcnt /= bcn_offs1) or (ttc_bcntres_dl = '0' and bcnt = bcn_offs1)) and en_bcnt_err = '1')then
				inc_bcnterr <= '1';
				ttc_bcnt_err <= '1';
			else
				inc_bcnterr <= '0';
			end if;
			if(ttc_brcst = x"7" or ttc_brcst = x"5")then
				ttc_soft_reset <= en_brcst;
			else
				ttc_soft_reset <= '0';
			end if;
			if(ttc_brcst = x"9")then
				ttc_start <= en_brcst;
			else
				ttc_start <= '0';
			end if;
			if(ttc_brcst = x"b")then
				ttc_stop <= en_brcst;
			else
				ttc_stop <= '0';
			end if;
			if(ttc_brcst = x"4")then
				brcst_GapTrig <= '1';
			elsif(rst_bcnt = '1')then
				brcst_GapTrig <= '0';
			end if;
			if(ttc_brcst = x"6")then
				brcst_GapPed <= '1';
			elsif(rst_bcnt = '1')then
				brcst_GapPed <= '0';
			end if;
			if(ttc_brcst = x"8")then
				brcst_GapLaser <= '1';
			else
				brcst_GapLaser <= '0';
			end if;
			if(ttc_OCres = '1')then
				calType <= x"0";
			elsif(brcst_GapLaser = '1')then
				if(Laser_TO(13) = '1')then
					calType <= x"1";
				elsif(calType /= x"f")then
					calType <= calType + 1;
				end if;
			end if;
			if(brcst_GapLaser = '1')then
				Laser_To <= (others => '0');
			elsif(Laser_TO(13) = '0' and ttc_bcntres_dl = '1')then
				Laser_To <= Laser_To + 1;
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(reset = '1')then
				en_bcnt_res <= '1';
			elsif(ttc_bcntres = '1' and bcn_off(12) = '1')then
				en_bcnt_res <= '0';
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
		if(strobe = '1')then
			if(reset = '1')then
				ttc_serr <= '0';
				ttc_derr <= '0';
				rate_div <= (others => '0');
				rate_cntr <= "000010";
				rate_OFW <= '0';
			else
				if(ttc_sinerrstr = '1')then
					ttc_serr <= '1';
				end if;
				if(ttc_dberrstr = '1')then
					ttc_derr <= '1';
				end if;
				if(ttc_sync_do_val = '1')then
					if(rate_div(7 downto 6) = "11" and rate_div(3) = '1')then
						dec_rate_cntr <= '1';
						rate_div <= (others => '0');
					else
						dec_rate_cntr <= '0';
						rate_div <= rate_div + 1;
					end if;
					if(dec_rate_cntr /= ttc_sync_do(7))then
						if(ttc_sync_do(7) = '1')then
							rate_cntr <= rate_cntr + 1;
						elsif(rate_cntr /= "000010")then
							rate_cntr <= rate_cntr - 1;
						end if;
					end if;
				end if;
				if(rate_cntr(5) = '1')then
					rate_OFW <= '1';
				elsif(rate_cntr(4) = '0')then
					rate_OFW <= '0';
				end if;
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(reset = '1')then
				event_number_avl_i <= '0';
			else
				event_number_avl_i <= ttc_l1accept_dl and run;
			end if;
			if(cal_winreg(31) = '0')then
				event_number(51 downto 48) <= x"0";
				event_number(44) <= '0';
			else
				event_number(51) <= cal_win and brcst_GapTrig and calType(3) and not brcst_GapPed;
				event_number(50) <= cal_win and brcst_GapTrig and calType(2) and not brcst_GapPed;
				event_number(49) <= cal_win and brcst_GapTrig and calType(1) and not brcst_GapPed;
				event_number(48) <= cal_win and ((brcst_GapTrig and calType(0)) or brcst_GapPed);
				event_number(44) <= cal_win and (brcst_GapTrig or brcst_GapPed);
			end if;
			event_number(59 downto 56) <= calType;
			event_number(55 downto 52) <= state;
			event_number(47) <= brcst_GapTrig;
			event_number(46) <= brcst_GapPed;
			event_number(45) <= cal_win;
			event_number(43 downto 0) <= oc & bcnt;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
			if(reset = '1')then
				sync_lost <= '0';
			elsif(evn_fifo_full = '1' and event_number_avl_i = '1')then
				sync_lost <= '1';
			end if;
		end if;
  end if;
end process;
-- i2c access is initiated by a write to the even address of the i2c space
-- bits 31-16 are ignored. Bit 15 is '1' for read and '0' for write.
-- Bits 14-13 are ignored. Bits 12-8 are address of the the i2c register file.
-- Bits 7-0 are data to written and ignored for read operation.
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			i2c_busy <= '0';
    elsif(strobe = '1')then
			if(we_byte(3) = '1' and TTC_i2c_addr(16 downto 2) = addr(14 downto 0))then
				i2c_busy <= '1';
			elsif(i2c_done = '1')then
				i2c_busy <= '0';
			end if;
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			i2c_done <= '0';
    elsif(ec_i2c = '1')then
			i2c_done <= eoc and (data_cycle or i2c_fail);
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
      if(TTC_i2c_addr(16 downto 2) = addr(14 downto 0) and i2c_busy = '0')then
				if(we_byte(2) = '1')then
					i2c_ad(15 downto 8) <= w_data(15 downto 8);
				end if;
				if(we_byte(3) = '1')then
					i2c_ad(7 downto 0) <= w_data(7 downto 0);
				end if;
      end if;
		end if;
  end if;
end process;
-- i2c state machine
-- since this is the only master, no STOP is generated
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			i2c_active <= '0';
			i2c_phase <= "000";
			data_cycle <= '0';
    elsif(ec_i2c = '1')then
			i2c_active <= i2c_busy;
			i2c_phase(2 downto 1) <= i2c_phase(1 downto 0);
			i2c_phase(0) <= i2c_busy and (i2c_phase(2) or not i2c_active);
			data_cycle <= i2c_active and ((eoc and i2c_phase(2)) or data_cycle);
		end if;
  end if;
end process;
-- scl generation and sda shift register 
process(clk)
begin
  if(clk'event and clk = '1')then
    if(ec_i2c = '1')then
      if(i2c_phase(1) = '1' and (i2c_cntr(4) = '0' or i2c_cntr(1) = '0'))then
				i2c_data(7 downto 0) <= i2c_data(6 downto 0) & T_sda;
			end if;
		end if;
  end if;
end process;
-- i2c_fail resets at the beginning of every i2c access
-- if acknoledge from slave is '1', i2c_fail is set and terminates the
-- operation
process(clk)
begin
  if(clk'event and clk = '1')then
    if(reset = '1')then
			i2c_fail <= '1';
    elsif(ec_i2c = '1')then
			if(i2c_busy = '1' and i2c_active = '0')then
				i2c_fail <= '0';
			elsif(T_sda = '1' and i2c_phase(1) = '1')then
				if(i2c_cntr = "01001" or
					(i2c_cntr = "10010" and (data_cycle = '0' or i2c_ad(15) = '0')))then
					i2c_fail <= '1';
				end if;
			end if;   
		end if;
  end if;
end process;
process(clk)
begin
  if(clk'event and clk = '1')then
    if(i2c_active = '0')then
			scl <= '1';
			sda <= (others => '1');
			eoc <= '0';
			i2c_cntr <= "00000";
    elsif(ec_i2c = '1')then
			scl <= (not i2c_phase(1) and scl) or i2c_phase(0) or eoc;
			if(i2c_phase(0) = '1' and i2c_cntr = "00000")then
				sda(17 downto 8) <=  '0' & ttc_id_i(5 downto 0) & data_cycle & (i2c_ad(15) and data_cycle) & '1';
				if(data_cycle = '0')then
					sda(7 downto 0) <= "000" & i2c_ad(12 downto 8);
				elsif(i2c_ad(15) = '1')then
					sda(7 downto 0) <= "11111111"; 
				else
					sda(7 downto 0) <= i2c_ad(7 downto 0); 
				end if;
			elsif(i2c_phase(2) = '1')then
				sda <= sda(16 downto 0) & '1';
			end if;
			if(i2c_phase(2) = '1')then
				if(i2c_cntr = "10010" or i2c_fail = '1')then
					eoc <= '1';
				else
					eoc <= '0';
				end if;
				if(eoc = '1')then
					i2c_cntr <= "00000";
				else
					i2c_cntr <= i2c_cntr + 1;
				end if;
			end if;
		end if;
  end if;
end process;
-- sda and scl outputs
T_sda <= '0' when sda(17) = '0' or eoc = '1' else 'Z';
T_scl <= '0' when scl = '0' else 'Z';
process(clk)
begin
  if(clk'event and clk = '1')then
    if(strobe = '1')then
      if(ttc_id_addr(16 downto 2) = addr(14 downto 0))then
				if(we_byte(0) = '1')then
					ttc_id_i(24) <= w_data(24);
				end if;
				if(we_byte(1) = '1')then
					ttc_id_i(23 downto 16) <= w_data(23 downto 16);
				end if;
				if(we_byte(2) = '1')then
					ttc_id_i(13 downto 8) <= w_data(13 downto 8);
				end if;
				if(we_byte(3) = '1')then
					ttc_id_i(7 downto 0) <= w_data(7 downto 0);
				end if;
      end if;
      if(TTC_cal_addr(16 downto 2) = addr(14 downto 0))then
				if(we_byte(0) = '1')then
					cal_winReg(31) <= w_data(31);
				end if;
				if(we_byte(1) = '1')then
					cal_winReg(21 downto 16) <= w_data(21 downto 16);
				end if;
				if(we_byte(3) = '1')then
					cal_winReg(5 downto 0) <= w_data(5 downto 0);
				end if;
      end if;
      if(TTC_bcnt_addr(16 downto 2) = addr(14 downto 0))then
				if(we_byte(2) = '1')then
					bcn_off(12 downto 8) <= w_data(12 downto 8);
				end if;
				if(we_byte(3) = '1')then
					bcn_off(7 downto 0) <= w_data(7 downto 0);
				end if;
      end if;
		end if;
  end if;
end process;
-- sda and scl outputs
T_sda <= '0' when sda(17) = '0' or eoc = '1' else 'Z';
T_scl <= '0' when scl = '0' else 'Z';
TTC_id <= ttc_id_i;
TTC_BCN_off <= bcn_off;
TTC_i2c_data <= i2c_data;
TTC_cal_win <= cal_winreg;
i2c_data(28) <= '0';
i2c_data(26 downto 8) <= "000" & x"00" & i2c_busy & i2c_fail & not i2c_ad(15) & "00000";
end ttc_arch;
